SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET row_security = off;

SET search_path = public, pg_catalog;

CREATE FUNCTION context_user(user_login character varying, user_name character varying) RETURNS text
    LANGUAGE sql
AS
$$
select set_config('loylabs.user_name', user_name, false);
select set_config('loylabs.user_login', user_login, false);
$$;

CREATE SEQUENCE history_id_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;

CREATE OR REPLACE FUNCTION trgf_all_history()
    RETURNS trigger AS
$BODY$
DECLARE
    OLDT    RECORD;
    stmtUpd text;
    stmtDel text;
BEGIN
    stmtUpd := 'insert into ' || TG_TABLE_NAME::TEXT || '_history values(nextval(''history_id_seq''), ($1).*)';
    stmtDel := 'insert into ' || TG_TABLE_NAME::TEXT || '_history values(nextval(''history_id_seq''), ($1).*)';

    if TG_OP = 'DELETE' then
        OLDT := OLD;
        OLDT.sys_operation := 'D';
        OLDT.sys_ds := CURRENT_TIMESTAMP;
        OLDT.sys_name := CURRENT_SETTING('loylabs.user_name');
        OLDT.sys_login := CURRENT_SETTING('loylabs.user_login');

        EXECUTE stmtUpd USING OLD;
        EXECUTE stmtDel USING OLDT;

        return OLD;
    elseif TG_OP = 'UPDATE' OR TG_OP = 'INSERT' then
        if TG_OP = 'UPDATE' then
            NEW.sys_operation := 'U';
            EXECUTE stmtUpd USING OLD;
        elseif TG_OP = 'INSERT' then
            NEW.sys_operation := 'I';
        end if;

        NEW.sys_ds := CURRENT_TIMESTAMP;
        NEW.sys_name := CURRENT_SETTING('loylabs.user_name');
        NEW.sys_login := CURRENT_SETTING('loylabs.user_login');

        RETURN NEW;
    end if;

    return null;
END;
$BODY$
    LANGUAGE plpgsql VOLATILE
                     COST 100;

CREATE SEQUENCE user_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;

CREATE TABLE users
(
    id                  BIGINT PRIMARY KEY       DEFAULT nextval('user_seq'),
    name                VARCHAR                                NOT NULL,
    email               VARCHAR                                NOT NULL,
    phone               VARCHAR(10),
    password            VARCHAR                                NOT NULL,
    registered          TIMESTAMP without time zone DEFAULT now() NOT NULL,
    enabled             BOOL                     DEFAULT TRUE  NOT NULL,
    parent_id           BIGINT references users (id),
    confirmed           BOOL                     DEFAULT FALSE NOT NULL,
    confirmation_code   VARCHAR(128),
    reset_password_code VARCHAR(128)
);
CREATE UNIQUE INDEX users_unique_email_idx ON users (email);
CREATE UNIQUE INDEX users_unique_phone_idx ON users (phone);

CREATE TABLE user_roles
(
    user_id BIGINT NOT NULL,
    role    VARCHAR(24),
    CONSTRAINT user_roles_idx UNIQUE (user_id, role),
    FOREIGN KEY (user_id) REFERENCES users (id) ON DELETE CASCADE
);

CREATE OR REPLACE FUNCTION trgf_users_fix_zero()
    RETURNS trigger AS
$BODY$
BEGIN
    IF 0 = NEW.parent_id THEN
        NEW.parent_id = null;
    END IF;

    IF NEW.password IS NULL THEN
        NEW.password = old.password;
    END IF;
    RETURN NEW;
END;
$BODY$
    LANGUAGE plpgsql VOLATILE
                     COST 100;

CREATE TRIGGER trg_users_fix_zero
    BEFORE INSERT OR UPDATE
    ON users
    FOR EACH ROW
EXECUTE PROCEDURE trgf_users_fix_zero();

CREATE SEQUENCE section_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE sections
(
    id            BIGINT PRIMARY KEY DEFAULT nextval('section_seq'),
    name          varchar(64) NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255),
    CONSTRAINT sections_name_idx UNIQUE (name)
);

CREATE TABLE sections_history
(
    history_id    bigint PRIMARY KEY NOT NULL,
    id            BIGINT             NOT NULL,
    name          varchar(64)        NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255)
);

CREATE TRIGGER trg_sections_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON sections
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE SEQUENCE topic_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE topics
(
    id            BIGINT PRIMARY KEY DEFAULT nextval('topic_seq'),
    section_id    BIGINT      NOT NULL,
    name          varchar(64) NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255),
    CONSTRAINT topics_name_idx UNIQUE (name),
    FOREIGN KEY (section_id) REFERENCES sections (id) ON DELETE CASCADE
);

CREATE TABLE topics_history
(
    history_id    bigint PRIMARY KEY NOT NULL,
    id            BIGINT             NOT NULL,
    section_id    BIGINT             NOT NULL,
    name          varchar(64)        NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255)
);

CREATE TRIGGER trg_topics_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON topics
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE SEQUENCE question_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE questions
(
    id            BIGINT PRIMARY KEY DEFAULT nextval('question_seq'),
    description   VARCHAR     NOT NULL,
    topic_id      BIGINT      NOT NULL,
    type          VARCHAR(32) NOT NULL,
    level         VARCHAR(32) NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255),
    FOREIGN KEY (topic_id) REFERENCES topics (id) ON DELETE CASCADE
);

CREATE TABLE questions_history
(
    history_id    bigint PRIMARY KEY NOT NULL,
    id            BIGINT             NOT NULL,
    description   VARCHAR            NOT NULL,
    topic_id      BIGINT             NOT NULL,
    type          VARCHAR(32)        NOT NULL,
    level         VARCHAR(32)        NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255)
);

CREATE TRIGGER trg_questions_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON questions
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE SEQUENCE answer_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE answers
(
    id            BIGINT PRIMARY KEY DEFAULT nextval('answer_seq'),
    question_id   BIGINT  NOT NULL,
    description   VARCHAR NOT NULL,
    correct       BOOL    NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255),
    FOREIGN KEY (question_id) REFERENCES questions (id) ON DELETE CASCADE
);

CREATE TABLE answers_history
(
    history_id    bigint PRIMARY KEY NOT NULL,
    id            BIGINT             NOT NULL,
    question_id   BIGINT             NOT NULL,
    description   VARCHAR            NOT NULL,
    correct       BOOL DEFAULT FALSE NOT NULL,
    sys_operation character varying(255),
    sys_ds        timestamp without time zone,
    sys_name      character varying(255),
    sys_login     character varying(255)
);

CREATE TRIGGER trg_answers_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON answers
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE SEQUENCE test_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE tests
(
    id              BIGINT PRIMARY KEY       DEFAULT nextval('test_seq'),
    name            VARCHAR(256)                           NOT NULL,
    description     VARCHAR,
    duration        INTEGER                                NOT NULL,
    user_id         BIGINT                                 NOT NULL,
    created         TIMESTAMP without time zone DEFAULT now() NOT NULL,
    level           VARCHAR(32)                            NOT NULL,
    question_number INTEGER                                NOT NULL,
    sys_operation   character varying(255),
    sys_ds          timestamp without time zone,
    sys_name        character varying(255),
    sys_login       character varying(255),
    FOREIGN KEY (user_id) REFERENCES users (id) ON DELETE CASCADE
);

CREATE TABLE tests_history
(
    history_id      bigint PRIMARY KEY                     NOT NULL,
    id              BIGINT                                 NOT NULL,
    name            VARCHAR(256)                           NOT NULL,
    description     VARCHAR,
    duration        INTEGER                                NOT NULL,
    user_id         BIGINT                                 NOT NULL,
    created         TIMESTAMP without time zone DEFAULT now() NOT NULL,
    level           VARCHAR(32)                            NOT NULL,
    question_number INTEGER                                NOT NULL,
    sys_operation   character varying(255),
    sys_ds          timestamp without time zone,
    sys_name        character varying(255),
    sys_login       character varying(255)
);

CREATE TRIGGER trg_tests_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON tests
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE TABLE test_questions
(
    test_id     BIGINT NOT NULL,
    question_id BIGINT NOT NULL,
    CONSTRAINT test_question_idx UNIQUE (test_id, question_id),
    FOREIGN KEY (test_id) REFERENCES tests (id) ON DELETE CASCADE,
    FOREIGN KEY (question_id) REFERENCES questions (id) ON DELETE CASCADE
);

CREATE SEQUENCE candidate_test_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;

CREATE TABLE candidate_tests
(
    id                      BIGINT PRIMARY KEY DEFAULT nextval('candidate_test_seq'),
    test_id                 BIGINT                   NOT NULL,
    link_code               VARCHAR(64)              NOT NULL,
    description             VARCHAR(1024)            NOT NULL,
    note                    VARCHAR(1024),
    candidate_name          VARCHAR(256)             NOT NULL,
    test_started            timestamp without time zone,
    test_finished           timestamp without time zone,
    status                  varchar(32)              NOT NULL,
    duration                INTEGER                  NOT NULL,
    created                 timestamp without time zone NOT NULL,
    user_id                 BIGINT                   NOT NULL,
    level                   VARCHAR(32)              NOT NULL,
    question_number         INTEGER                  NOT NULL,
    correct_question_number INTEGER,

    FOREIGN KEY (user_id) REFERENCES users (id) ON DELETE CASCADE
);
CREATE UNIQUE INDEX candidate_tests_link_code_idx ON candidate_tests (link_code);

CREATE SEQUENCE candidate_question_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;

CREATE TABLE candidate_questions
(
    id                BIGINT PRIMARY KEY DEFAULT nextval('candidate_question_seq'),
    candidate_test_id BIGINT                           NOT NULL,
    description       VARCHAR                          NOT NULL,
    topic_name        varchar(64)                      NOT NULL,
    type              VARCHAR(10)                      NOT NULL,
    answered          BOOL               DEFAULT FALSE NOT NULL,
    CONSTRAINT candidate_test_question_num_idx UNIQUE (candidate_test_id, description),
    FOREIGN KEY (candidate_test_id) REFERENCES candidate_tests (id) ON DELETE CASCADE
);

CREATE SEQUENCE candidate_answer_seq
    INCREMENT 1
    MINVALUE 1
    MAXVALUE 9223372036854775807
    START 1
    CACHE 1;
CREATE TABLE candidate_answers
(
    id                    BIGINT PRIMARY KEY DEFAULT nextval('candidate_answer_seq'),
    candidate_question_id BIGINT                           NOT NULL,
    description           VARCHAR                          NOT NULL,
    correct               BOOL               DEFAULT FALSE NOT NULL,
    selected              BOOL               DEFAULT FALSE NOT NULL,
    sys_operation         character varying(255),
    sys_ds                timestamp without time zone,
    sys_name              character varying(255),
    sys_login             character varying(255),
    FOREIGN KEY (candidate_question_id) REFERENCES candidate_questions (id) ON DELETE CASCADE
);

CREATE TABLE sys_params
(
    smtp_host                           character varying(255),
    smtp_port                           INTEGER,
    smtp_user                           character varying(255),
    smtp_password                       character varying(1024),
    smtp_email                          character varying(255),
    email_name                          character varying(255),
    email_subject                       character varying(512),
    email_register_confirmation_message character varying(1024),
    email_reset_password_message        character varying(1024),
    email_finish_test_message           character varying(1024),
    default_test_balance                INTEGER                      default 0,
    default_user_limit                  INTEGER                      default 0,
    sys_params_id                       INTEGER PRIMARY KEY NOT NULL default 1,
    sys_operation                       character varying(255),
    sys_ds                              timestamp without time zone,
    sys_name                            character varying(255),
    sys_login                           character varying(255),
    CONSTRAINT single_row_constraint CHECK (sys_params_id = 1)

);

CREATE TABLE sys_params_history
(
    history_id                          bigint PRIMARY KEY NOT NULL,
    smtp_host                           character varying(255),
    smtp_port                           INTEGER,
    smtp_user                           character varying(255),
    smtp_password                       character varying(1024),
    smtp_email                          character varying(255),
    email_name                          character varying(255),
    email_subject                       character varying(512),
    email_register_confirmation_message character varying(1024),
    email_reset_password_message        character varying(1024),
    email_finish_test_message           character varying(1024),
    default_test_balance                INTEGER,
    default_user_limit                  INTEGER,
    sys_params_id                       INTEGER,
    sys_operation                       character varying(255),
    sys_ds                              timestamp without time zone,
    sys_name                            character varying(255),
    sys_login                           character varying(255)
);

INSERT INTO sys_params (sys_params_id)
VALUES (1);

CREATE TRIGGER trg_sys_params_history
    BEFORE INSERT OR UPDATE OR DELETE
    ON sys_params
    FOR EACH ROW
EXECUTE PROCEDURE trgf_all_history();

CREATE TABLE accounts
(
    user_id      BIGINT PRIMARY KEY,
    test_balance INTEGER,
    user_limit   INTEGER,
    FOREIGN KEY (user_id) REFERENCES users (id) ON DELETE CASCADE
);
