package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.ListResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.CandidateAnswerDto;
import com.example.new_test_lab.dto.CandidateQuestionDto;
import com.example.new_test_lab.dto.CandidateTestDto;
import com.example.new_test_lab.dto.CreateCandidateTestDto;
import com.example.new_test_lab.dto.CreateResultCodeDto;
import com.example.new_test_lab.dto.TopicStatDto;
import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.model.ResultCode;
import com.example.new_test_lab.model.User;
import com.example.new_test_lab.service.CandidateQuestionService;
import com.example.new_test_lab.service.CandidateTestService;
import com.example.new_test_lab.service.ResultCodeService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.countPages;
import static com.example.new_test_lab.util.AppUtil.isAnsweredCorrect;
import static com.example.new_test_lab.util.SecurityUtil.authUser;
import static com.example.new_test_lab.util.SecurityUtil.authUserId;
import static com.example.new_test_lab.util.TestUtil.fromDto;
import static java.util.stream.Collectors.toList;


@Slf4j
@RestController
@RequestMapping("/candidate-tests")
public class CandidateTestController {

    private CandidateTestService testService;

    private CandidateQuestionService questionService;

    private ResultCodeService resultCodeService;

    @Autowired
    public CandidateTestController(CandidateTestService testService,
                                   CandidateQuestionService questionService,
                                   ResultCodeService resultCodeService) {
        this.testService = testService;
        this.questionService = questionService;
        this.resultCodeService = resultCodeService;
    }

    @ApiOperation("Создание теста для кандидата")
    @PostMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid CreateCandidateTestDto candidateTestDto) {
        log.info("#create candidate test: {}", candidateTestDto);

        return new SuccessResponse(testService.create(fromDto(candidateTestDto), authUserId()));
    }

    @ApiOperation("Получение всех тестов кандидатов для аутентифицированного пользователя с фильтрацией")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "level", example = "JUNIOR, MIDDLE",
                    value = "Фильтрация по уровню. Доступные уровни: JUNIOR, MIDDLE, SENIOR"),
            @ApiImplicitParam(name = "position", example = "BACKEND, FRONTEND",
                    value = "Фильтрация по должности. Доступные должности: BACKEND, FRONTEND, DESIGN, QA"),
            @ApiImplicitParam(name = "status", example = "FINISHED",
                    value = "Фильтрация по статусу. Доступные статусы: NEW, IN_PROGRESS, FINISHED"),
            @ApiImplicitParam(name = "isArchived", example = "true",
                    value = "Фильтрация по архивированию. Если NULL - то отображаются все." +
                            "Если TRUE - только архивированные, если FALSE - только неархивированные")
    })
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll(@RequestParam(required = false, defaultValue = "0") int page,
                         @RequestParam(required = false, defaultValue = "10") int size,
                         @RequestParam(required = false, defaultValue = "created,desc") String sort,
                         @RequestParam(required = false) String level,
                         @RequestParam(required = false) String position,
                         @RequestParam(required = false) String status,
                         @RequestParam(required = false) Boolean isArchived) {
        log.info("#get all");

        User authUser = authUser();
        List<CandidateTest> tests = testService.getAll(authUser.getId(), page, size, sort, level, position, status, isArchived);
        int count = testService.countAll(authUser.getId(), level, position, status, isArchived);
        return new ListResponse(tests, countPages(count, size), count);
    }

    @ApiOperation("Получение теста кандидата для аутентифицированного пользователя по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);

        CandidateTest candidateTest = testService.get(id, authUserId());
        if (candidateTest.isNotViewed()) {
            testService.setViewed(candidateTest, authUserId());
        }
        List<TopicStatDto> topicStats = testService.getTopicStatistic(candidateTest.getId());
        if (candidateTest.getCorrectQuestionNumber() == null)
            candidateTest.setCorrectQuestionNumber(topicStats.stream()
                    .map(TopicStatDto::getCorrectQuestions)
                    .mapToInt(Integer::intValue)
                    .sum());
        return new SuccessResponse(new CandidateTestDto(candidateTest, topicStats));
    }

    @ApiOperation("Пометка теста кандидата как архивированного")
    @PostMapping(value = "/{id}/archive", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object archive(@PathVariable int id) {
        log.info("#archive by id: {}", id);

        testService.archive(id, authUserId());
        return new SuccessResponse();
    }

    @ApiOperation("Разархивация теста кандидата")
    @PostMapping(value = "/{id}/unarchive", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object unarchive(@PathVariable int id) {
        log.info("#unarchive by id: {}", id);

        testService.unarchive(id, authUserId());
        return new SuccessResponse();
    }

    @ApiOperation("Удаление теста кандидата для аутентифицированного пользователя по ID")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        testService.delete(id, authUserId());
        return new SuccessResponse();
    }

    @ApiOperation("Получение вопросов теста кандидата для аутентифицированного пользователя по ID")
    @GetMapping(value = "/{id}/questions", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getQuestions(@PathVariable int id,
                               @RequestParam(required = false, defaultValue = "0") int page,
                               @RequestParam(required = false, defaultValue = "10") int size) {

        log.info("#get questions by id: {}", id);

        CandidateTest candidateTest = testService.get(id, authUserId());
        List<CandidateQuestion> questions = questionService.getQuestionsByTestId(candidateTest.getId(), page, size);
        Map<Long, List<CandidateAnswer>> answersMap = questionService
                .getAnswers(questions.stream().mapToLong(CandidateQuestion::getId).toArray())
                .stream()
                .collect(Collectors.groupingBy(CandidateAnswer::getCandidateQuestionId));

        return new ListResponse(questions
                .stream().map(q -> new CandidateQuestionDto(q,
                        answersMap.get(q.getId())
                                .stream().map(ca -> new CandidateAnswerDto(ca, true)).collect(toList()),
                        isAnsweredCorrect(answersMap.get(q.getId()), q.getType())))
                .collect(toList()), candidateTest.getQuestionNumber() / size,
                candidateTest.getQuestionNumber());
    }

    @ApiOperation("Создание кода для получения результатов завершённого теста кандидата")
    @PostMapping(value = "/result-code",
            produces = MediaType.APPLICATION_JSON_VALUE,
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object generateResultCode(@RequestBody @Valid CreateResultCodeDto resultCodeDto) {
        log.info("#create result code for test with id: {}", resultCodeDto.getCandidateTestId());

        ResultCode resultCode = resultCodeService.create(resultCodeDto);
        return new SuccessResponse(resultCode);
    }

    @ApiOperation("Получение существующих кодов получения результатов для завершённого теста кандидата по ID")
    @GetMapping(value = "/{id}/result-code",
            produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAllResultCodes(@PathVariable long id) {
        log.info("#get result code for test with id: {}", id);

        List<ResultCode> resultCodes = resultCodeService.findAllWithCandidateTestId(id, authUserId());
        return new ListResponse(resultCodes, 1, resultCodes.size());
    }

    @ApiOperation("Получение существующих кодов получения результатов для завершённого теста")
    @GetMapping(value = "/result-code",
            produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAllResultCodes() {
        log.info("#get result code");

        List<ResultCode> resultCodes = resultCodeService.findAll(authUserId());
        return new ListResponse(resultCodes, 1, resultCodes.size());
    }

}
