package com.example.new_test_lab.controller;



import com.example.new_test_lab.controller.response.ListResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.CreateOrderDto;
import com.example.new_test_lab.dto.DocumentsRequestDto;
import com.example.new_test_lab.event.OrderStatusChangedEvent;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.User;
import com.example.new_test_lab.model.enums.OrderStatus;
import com.example.new_test_lab.repository.SysParamsRepository;
import com.example.new_test_lab.service.OrderService;
import com.example.new_test_lab.tinkoff.TinkoffParams;
import com.example.new_test_lab.tinkoff.TinkoffSignUtil;
import com.example.new_test_lab.tinkoff.request.CallbackRequest;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.crypto.encrypt.TextEncryptor;
import org.springframework.web.bind.annotation.*;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Paths;
import java.security.NoSuchAlgorithmException;

import static com.example.new_test_lab.exception.Exceptions.BAD_REQUEST;
import static com.example.new_test_lab.exception.Exceptions.CRYPTO_ERROR;
import static com.example.new_test_lab.util.AppUtil.countPages;
import static com.example.new_test_lab.util.OrderUtil.resolveStatus;
import static com.example.new_test_lab.util.SecurityUtil.authUser;
import static com.example.new_test_lab.util.SecurityUtil.authUserId;


@Slf4j
@RestController
@RequestMapping("/orders")
@RequiredArgsConstructor
public class OrderController {

    private OrderService orderService;
    private ApplicationEventPublisher applicationEventPublisher;
    private SysParamsRepository sysParamsRepository;
    private TextEncryptor encryptor;

    public OrderController(OrderService orderService, ApplicationEventPublisher applicationEventPublisher, SysParamsRepository sysParamsRepository, TextEncryptor encryptor) {
        this.orderService = orderService;
        this.applicationEventPublisher = applicationEventPublisher;
        this.sysParamsRepository = sysParamsRepository;
        this.encryptor = encryptor;
    }

    @ApiOperation( "Создание заказа")
    @PostMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object createOrder(@RequestBody CreateOrderDto createOrderDto) {
        log.info("#Create order:  {}", createOrderDto);

        return orderService.createOrder(createOrderDto.getRateId(), authUserId(), createOrderDto.getPaymentMethod());
    }

    @ApiOperation("Получение всех заказов" )
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getOrders(
            @RequestParam(required = false, defaultValue = "0") int page,
            @RequestParam(required = false, defaultValue = "10") int size,
            @RequestParam(required = false, defaultValue = "order_date,desc") String sort
    ) {
        User authUser = authUser();
        log.info("#get all for {}", authUser.getEmail());
        int count = orderService.countAll(authUser.getId());
        return new ListResponse(orderService.getAll(authUser.getId(), page, size, sort), countPages(count, size), count);
    }

    @ApiOperation( "Получение заказа по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);
        return new SuccessResponse(orderService.get(id, authUserId()));
    }

    @ApiOperation( "Колл-бек на создание заказа Тинькофф")
    @PostMapping(value = "status/tinkoff", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object onOrderStatusChanged(@RequestBody CallbackRequest request) {
        log.info("Tinkoff order status callback received: " + request);

        TinkoffParams params = sysParamsRepository.get();
        String password = encryptor.decrypt(params.getTinkoffPassword());
        try {
            if (TinkoffSignUtil.validateSign(request, password)) {
                OrderStatus orderStatus = resolveStatus(request.getStatus());
                OrderStatusChangedEvent event = new OrderStatusChangedEvent(this, Long.parseLong(request.getOrderId()), request.getPaymentId(), orderStatus);
                applicationEventPublisher.publishEvent(event);
            } else {
                log.error("Token didn't pass validation");
            }
        } catch (NoSuchAlgorithmException e) {
            throw new ServerException(CRYPTO_ERROR);
        }

        return "OK";
    }

    @ApiOperation("Запрос на получение документов" )
    @PostMapping(value = "/documents", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object requestDocuments(@RequestBody DocumentsRequestDto documentsRequest) {
        User user = authUser();
        log.info("#request documents for user id: {}", user.getId());
        orderService.requestDocuments(documentsRequest, user.getId());
        return new SuccessResponse();
    }

    @ApiOperation("Запрос на получение чека по ID" )
    @GetMapping(value = "/invoice/{orderId}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getInvoice(@PathVariable int orderId) {
        log.info("#get invoice by id: {}", orderId);
        try {
            String path = orderService.downloadInvoice(orderId, authUser().getCompanyInfoId());
            File file = new File(path);
	        InputStreamResource resource = new InputStreamResource(new FileInputStream(file));
            return ResponseEntity.ok()
                                 .contentLength(file.length())
                                 .contentType(MediaType.APPLICATION_OCTET_STREAM)
                                 .body(resource);
        } catch (FileNotFoundException ex) {ex.printStackTrace();}
        throw new ServerException(BAD_REQUEST);
    }

    @ApiOperation("Запрос на получение оферты" )
    @GetMapping(value = "/offer", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getOffer() {
        log.info("#get offer");
        try {
	        URL url = getClass().getClassLoader().getResource("offer.pdf");
	        File file = Paths.get(url.toURI()).toFile();
            InputStreamResource resource = new InputStreamResource(new FileInputStream(file));
            return ResponseEntity.ok()
                             .contentLength(file.length())
                             .contentType(MediaType.APPLICATION_OCTET_STREAM)
                             .body(resource);
        } catch (IOException | URISyntaxException ex) {ex.printStackTrace();}
        throw new ServerException(BAD_REQUEST);
    }
}
