package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.ListResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.QuestionDto;
import com.example.new_test_lab.model.Answer;
import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.model.enums.Position;
import com.example.new_test_lab.model.enums.QuestionType;
import com.example.new_test_lab.service.QuestionService;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.text.WordUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Stream;

import static com.example.new_test_lab.util.AppUtil.countPages;
import static com.example.new_test_lab.util.AppUtil.toDto;


@Slf4j
@RestController
public class QuestionController {

    private QuestionService questionService;

    @Autowired
    public QuestionController(QuestionService questionService) {
        this.questionService = questionService;
    }

    @ApiOperation("Получение вопроса по ID")
    @GetMapping(value = "/questions/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by quiestion id: {}", id);

        Question question = questionService.get(id);
        List<Answer> answers = questionService.getAnswers(id);
        return new SuccessResponse(toDto(question, answers));
    }

    @ApiOperation("Получение всех вопросов в категории")
    @GetMapping(value = "topics/{id}/questions", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll(@PathVariable int id,
                         @RequestParam(required = false, defaultValue = "0") int page,
                         @RequestParam(required = false, defaultValue = "10") int size,
                         @RequestParam(required = false, defaultValue = "id,asc") String sort) {
        log.info("#get all questions by topic id: {}", id);

        List<Question> questions = questionService.getAllByTopicId(id, page, size, sort);
        int count = questionService.countAllByTopicId(id);
        return new ListResponse(questions, countPages(count, size), count);
    }

    @ApiOperation("Создание вопроса")
    @PostMapping(value = "/questions", produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid QuestionDto question) {
        log.info("#create: {}", question);

        questionService.create(question.getQuestion(), question.getAnswers());
        return new SuccessResponse();
    }

    @ApiOperation("Изменение вопроса")
    @PutMapping(value = "/questions", produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid QuestionDto question) {
        log.info("#update: {}", question);

        questionService.update(question.getQuestion(), question.getAnswers());
        return new SuccessResponse();
    }

    @ApiOperation("Удаление вопроса по ID")
    @DeleteMapping(value ="/questions/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        questionService.delete(id);
        return new SuccessResponse();
    }

    @ApiOperation("Получение уровней вопросов")
    @GetMapping(value ="/questions/levels", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object levels() {
        return new SuccessResponse(Stream.of(Level.values()).map(l -> WordUtils.capitalizeFully(l.getCode())).toArray());
    }

    @ApiOperation("Получение должностей вопросов")
    @GetMapping(value ="/questions/positions", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object positions() {
        return new SuccessResponse(Stream.of(Position.values()).map(l -> WordUtils.capitalizeFully(l.getCode())).toArray());
    }

    @ApiOperation("Получение типов вопросов")
    @GetMapping(value ="/questions/types", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object types() {
        return new SuccessResponse(Stream.of(QuestionType.values()).map(l -> WordUtils.capitalizeFully(l.getCode())).toArray());
    }
}
