package com.example.new_test_lab.controller;

import com.example.new_test_lab.controller.response.ListResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.CountRequestDto;
import com.example.new_test_lab.dto.TestDto;
import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.model.Test;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.model.enums.Position;
import com.example.new_test_lab.service.QuestionService;
import com.example.new_test_lab.service.TestService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.text.WordUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;


import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.example.new_test_lab.util.AppUtil.countPages;
import static com.example.new_test_lab.util.SecurityUtil.authUserId;
import static com.example.new_test_lab.util.TestUtil.fillIn;


@Slf4j
@RestController
@RequestMapping("/tests")
public class TestController {

    private TestService testService;
    private QuestionService questionService;

    @Autowired
    public TestController(TestService testService, QuestionService questionService) {
        this.testService = testService;
        this.questionService = questionService;
    }

    @ApiOperation( "Получение теста по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);

        Test test = testService.get(id, authUserId());
        List<Long> topicIds = questionService.getAllByTestId(id).stream().map(Question::getTopicId).distinct().collect(Collectors.toList());
        return new SuccessResponse(new TestDto(test, topicIds));
    }

    @ApiOperation( "Получение всех тестов")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "level", example = "JUNIOR, MIDDLE",
                              value= "Уровень для фильтрации. Доступные уровни: JUNIOR, MIDDLE, SENIOR"),
            @ApiImplicitParam(name = "position", example = "BACKEND, FRONTEND",
                              value = "Уровень для фильтрации. Доступные уровни: BACKEND, FRONTEND, DESIGN, QA")
    })
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getAll(@RequestParam(required = false, defaultValue = "0") int page,
                         @RequestParam(required = false, defaultValue = "10") int size,
                         @RequestParam(required = false, defaultValue = "created,desc") String sort,
                         @RequestParam(required = false) String level,
                         @RequestParam(required = false) String position) {
        log.info("#get all");

        long authId = authUserId();
        List<Test> tests = testService.getAll(authId, page, size, sort, level, position);
        int count = testService.countAll(authId, level, position);
        return new ListResponse(tests, countPages(count, size), count);
    }

    @ApiOperation( "Создание теста")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object create(@RequestBody @Valid TestDto request) {
        log.info("#create test: {}", request.getTest());

        testService.create(request.getTest(), request.getTopicIds(), authUserId());
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление теста по ID")
    @PutMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid Test test) {
        log.info("#update test: {}", test);

        long authId =  authUserId();

        Test testDb = testService.get(test.getId(), authUserId());
        fillIn(test, testDb);
        testService.update(testDb, authId);
        return new SuccessResponse();
    }

    @ApiOperation( "Удаление теста по ID")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        testService.delete(id, authUserId());
        return new SuccessResponse();
    }

    @ApiOperation( "Подсчёт вопросов по топику")
    @PostMapping(value = "/count", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object countQuestionsByTopics(@RequestBody CountRequestDto countRequestDto) {
        log.info("#count questions by topicIds: {}", countRequestDto.getTopicIds());

        return testService.countQuestionsByTopicIds(countRequestDto.getTopicIds());
    }

    @ApiOperation( "Получение уровней тестов")
    @GetMapping(value ="/levels", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object levels() {
        return new SuccessResponse(Stream.of(Level.values()).map(l -> WordUtils.capitalizeFully(l.getCode())).toArray());
    }


    @ApiOperation( "Получение должностей тестов")
    @GetMapping(value ="/positions", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object positions() {
        return new SuccessResponse(Stream.of(Position.values()).map(l -> WordUtils.capitalizeFully(l.getCode())).toArray());
    }

}
