package com.example.new_test_lab.controller;


import com.example.new_test_lab.controller.response.FailResponse;
import com.example.new_test_lab.controller.response.SuccessResponse;
import com.example.new_test_lab.dto.BalanceDto;
import com.example.new_test_lab.dto.PasswordDto;
import com.example.new_test_lab.dto.UserDto;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.model.Account;


import com.example.new_test_lab.model.User;
import com.example.new_test_lab.service.UserService;
import com.example.new_test_lab.util.UserUtil;
import io.swagger.annotations.ApiOperation;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.time.ZoneId;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.UserUtil.toDto;
import static com.example.new_test_lab.util.SecurityUtil.authUser;
import static com.example.new_test_lab.util.SecurityUtil.authUserId;
import static com.example.new_test_lab.util.UserUtil.fromDto;



@Slf4j
@RestController
@RequestMapping("/users")
public class UserController {

    private UserService service;

    @Autowired
    public UserController(UserService service) {
        this.service = service;
    }

    @ApiOperation( "Получение пользователя по ID")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object get(@PathVariable int id) {
        log.info("#get by id: {}", id);
        return new SuccessResponse(toDto(service.get(id, authUserId())));
    }

    @ApiOperation( "Получение баланса пользователя по ID")
    @GetMapping(value = "/balance", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getBalance() {
        long userId = authUserId();
        log.info("#getBalance by user id {}", userId);
        Account account = service.getAccount(userId);
        return new SuccessResponse(new BalanceDto(account.getTestBalance(), account.getUserLimit(), Date.from(account.getSubscriptionEnd().atZone(
                ZoneId.systemDefault()).toInstant())));
    }

    @ApiOperation( "Получение аутентифицированного пользователя")
    @GetMapping(value = "/profile", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object getProfile() {
        User user = authUser();
        long userId = authUserId();
        log.info("#getProfile by user id {}", userId);

        if (user == null) return new FailResponse(Exceptions.EMAIL_NOT_CONFIRMED);
        // this breaks registration
        // to confirm registration, you need to login, and you can't login if you're not confirmed
//        if (user == null || !user.isConfirmed()) return new FailResponse(Exceptions.EMAIL_NOT_CONFIRMED);

        return new SuccessResponse(toDto(service.get(userId, userId)));
    }

    @ApiOperation( "Получение всех пользователей")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public Object list() {
        log.info("#getList");

        List<UserDto> users = service.getAll(authUserId()).stream().map(UserUtil::toDto).collect(Collectors.toList());
        return new SuccessResponse(users);
    }

    @ApiOperation( "Регистрация пользователя")
    @PostMapping(value ="/register", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object register(@RequestBody @Valid UserDto userDto) {
        log.info("#register user: {}", userDto);

        Long authId = authUserId();
        return new SuccessResponse(UserUtil.toDto(service.create(fromDto(userDto), authId)));
    }

    @ApiOperation( "Подтверждение регистрации")
    @PutMapping(value ="/register", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object confirm(@RequestParam String confirm) {
        log.info("#confirm registration: {}", confirm);

        Long authId = authUserId();
        service.confirm(confirm, authId);
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление пользователя по ID")
    @PutMapping(produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    public Object update(@RequestBody @Valid UserDto userDto) {
        log.info("#update user: {}", userDto);

        Long authId = authUserId();
        User user = service.get(userDto.getId(), authId);
        service.update(fromDto(userDto, user), authUserId());
        return new SuccessResponse(toDto(user));
    }

    @ApiOperation( "Удаление пользователя по ID")
    @DeleteMapping(value ="/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object delete(@PathVariable int id) {
        log.info("#delete by id: {}", id);

        service.delete(id, authUserId());
        return new SuccessResponse();
    }

    @ApiOperation( "Создание кода сброса пароля")
    @PutMapping(value ="/reset-password", produces = MediaType.APPLICATION_JSON_VALUE)
    public Object resetPassword(@RequestParam String email) {
        log.info("#reset password for email: {}", email);

        service.resetPassword(email);
        return new SuccessResponse();
    }

    @ApiOperation( "Обновление пароля по коду сброса пароля")
    @PostMapping(value ="/reset-password", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public Object setNewPassword(@RequestParam String resetCode, @RequestBody @Valid PasswordDto password) {
        log.info("#set new password by resetCode: {}", resetCode);

        service.setNewPassword(resetCode, password.getPassword());
        return new SuccessResponse();
    }
}
