package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.repository.CandidateTestRepository;
import com.google.common.primitives.Longs;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.sql.Types;
import java.util.Arrays;
import java.util.List;

import static com.example.new_test_lab.util.AppUtil.*;


@Repository
public class CandidateTestRepositoryImpl implements CandidateTestRepository {
    private static final RowMapper<CandidateTest> ROW_MAPPER = BeanPropertyRowMapper.newInstance(CandidateTest.class);

    private static final String DEFAULT_SORT_BY = "created";
    private static final List<String> ALLOWED_COLUMN_NAMES = Arrays.asList("id", "note", "description", "duration", "created", "level", "position", "test_started", "test_finished", "candidate_name", "not_viewed", "name", "archived");

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public CandidateTestRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("candidate_tests")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public boolean delete(long id, long userId) {
        return jdbcTemplate.update("DELETE FROM candidate_tests WHERE id=? AND user_id=?", id, userId) != 0;
    }

    @Override
    public boolean archive(long id, long userId) {
        return jdbcTemplate.update("UPDATE candidate_tests SET archived = TRUE WHERE id=? AND user_id=?", id, userId) != 0;
    }

    @Override
    public boolean unarchive(long id, long userId) {
        return jdbcTemplate.update("UPDATE candidate_tests SET archived = FALSE WHERE id=? AND user_id=?", id, userId) != 0;
    }

    @Override
    public CandidateTest save(CandidateTest test, long userId) {
        test.setUserId(userId);
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(test);
        source.registerSqlType("status", Types.VARCHAR);
        source.registerSqlType("level", Types.VARCHAR);
        if (test.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            test.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE candidate_tests " +
                            " SET test_id=:testId, link_code=:linkCode, description=:description, name=:name, note=:note, created=:created, candidate_name=:candidateName, " +
                            " test_started=:testStarted, test_finished=:testFinished, status=:status, duration=:duration, correct_question_number=:correctQuestionNumber, " +
                            " question_number=:questionNumber, level=:level, not_viewed=:notViewed, increased_complexity=:increasedComplexity, archived=:archived" +
                            " WHERE id=:id AND user_id=:userId"
                    , source) == 0) {
                return null;
            }
        }
        return test;
    }

    @Override
    public CandidateTest getByCode(String code) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM candidate_tests WHERE link_code=?", ROW_MAPPER, code));
    }

    @Override
    public CandidateTest getById(long id) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("id", id);

        return DataAccessUtils.singleResult(namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_tests WHERE id =:id", parameters, ROW_MAPPER));
    }

    @Override
    public CandidateTest getByIdAndUserIds(long id, long... userIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("id", id);
        parameters.addValue("userIds", Longs.asList(userIds));

        return DataAccessUtils.singleResult(namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_tests WHERE id =:id AND user_id IN (:userIds)", parameters, ROW_MAPPER));
    }

    @Override
    public List<CandidateTest> getAll(long... userIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userIds", Longs.asList(userIds));
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_tests WHERE user_id IN (:userIds)", parameters, ROW_MAPPER);
    }

    @Override
    public List<CandidateTest> getAll(int page, int size, String sort, long... userIds) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userIds", Longs.asList(userIds));
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM candidate_tests WHERE user_id IN (:userIds) ORDER BY " + sortBy + " OFFSET :offset LIMIT :size", parameters, ROW_MAPPER);
    }

    @Override
    public List<CandidateTest> getAll(int page, int size, String sort, String levels, String position, String status, Boolean isArchived, long... userIds) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        List<String> levelFilter = getLevelsFilter(levels);
        List<String> positionFilter = getPositionsFilter(position);
        List<String> statusesFilter = getStatusesFilter(status);

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userIds", Longs.asList(userIds));
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);

        StringBuilder queryBuilder = new StringBuilder();
        queryBuilder.append("SELECT * FROM candidate_tests WHERE user_id IN (:userIds)");
        if (levelFilter != null && !levelFilter.isEmpty()) {
            parameters.addValue("levels", levelFilter);
            queryBuilder.append(" AND level IN (:levels)");
        }
        if (positionFilter != null && !positionFilter.isEmpty()) {
            parameters.addValue("positions", positionFilter);
            queryBuilder.append(" AND position IN (:positions)");
        }
        if (statusesFilter != null && !statusesFilter.isEmpty()) {
            parameters.addValue("statuses", statusesFilter);
            queryBuilder.append(" AND status IN (:statuses)");
        }
        if (isArchived != null) {
            parameters.addValue("archived", isArchived);
            queryBuilder.append(" AND archived = :archived");
        }
        queryBuilder.append(" ORDER BY ").append(sortBy).append(" OFFSET :offset LIMIT :size");

        return namedParameterJdbcTemplate.query(queryBuilder.toString(), parameters, ROW_MAPPER);
    }

    @Override
    public int countAll(long... userIds) {
        SqlParameterSource parameters = new MapSqlParameterSource("userIds", Longs.asList(userIds));
        return namedParameterJdbcTemplate.queryForObject("SELECT count(1) FROM candidate_tests WHERE user_id IN (:userIds)", parameters, Integer.class);
    }

    @Override
    public int countAll(String levels, String positions, String status, Boolean isArchived, long... userIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource("userIds", Longs.asList(userIds));
        List<String> levelFilter = getLevelsFilter(levels);
        List<String> positionFilter = getPositionsFilter(positions);
        List<String> statusesFilter = getStatusesFilter(status);

        StringBuilder queryBuilder = new StringBuilder();
        queryBuilder.append("SELECT count(1) FROM candidate_tests WHERE user_id IN (:userIds)");
        if (levelFilter != null && !levelFilter.isEmpty()) {
            parameters.addValue("levels", levelFilter);
            queryBuilder.append(" AND level IN (:levels)");
        }
        if (positionFilter != null && !positionFilter.isEmpty()) {
            parameters.addValue("positions", positionFilter);
            queryBuilder.append(" AND position IN (:positions)");
        }
        if (statusesFilter != null && !statusesFilter.isEmpty()) {
            parameters.addValue("statuses", statusesFilter);
            queryBuilder.append(" AND status IN (:statuses)");
        }
        if (isArchived != null) {
            parameters.addValue("archived", isArchived);
            queryBuilder.append(" AND archived = :archived");
        }

        return namedParameterJdbcTemplate.queryForObject(queryBuilder.toString(), parameters, Integer.class);
    }

    @Override
    public List<CandidateTest> getExpired() {
        return jdbcTemplate.query(
                "SELECT * FROM candidate_tests WHERE status=? AND test_started + (duration * interval '1' second) < now()", ROW_MAPPER, CandidateTestStatus.IN_PROGRESS.name());
    }

    @Override
    public boolean updateStatus(long testId, CandidateTestStatus status) {
        return jdbcTemplate.update("UPDATE candidate_tests SET status=? WHERE id=?", status.name(), testId) != 0;
    }
}
