package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Order;
import com.example.new_test_lab.model.OrderView;
import com.example.new_test_lab.repository.OrderRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.sql.Types;
import java.util.Arrays;
import java.util.List;

import static com.example.new_test_lab.util.AppUtil.getDirection;
import static com.example.new_test_lab.util.AppUtil.getSortColumnName;


@Repository
public class OrderRepositoryImpl implements OrderRepository {
    private static final String DEFAULT_SORT_BY = "order_date";
    private static final List<String> ALLOWED_COLUMN_NAMES = Arrays.asList("id", "order_date", "payment_date", "duration", "amount", "order_status");
    private static final BeanPropertyRowMapper<Order> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Order.class);
    private static final BeanPropertyRowMapper<OrderView> VIEW_ROW_MAPPER = BeanPropertyRowMapper.newInstance(OrderView.class);

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public OrderRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("orders")
                .usingGeneratedKeyColumns("id");
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public List<OrderView> getAll(long userId, int page, int size, String sort) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userId", userId);
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        parameters.addValue("sort", sortBy);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM orders_view WHERE user_id=:userId ORDER BY " + sortBy + " OFFSET :offset LIMIT :size ", parameters, VIEW_ROW_MAPPER);
    }

    @Override
    public List<OrderView> getAll(int page, int size, String sort) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        parameters.addValue("sort", sortBy);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM orders_view ORDER BY " + sortBy + " OFFSET :offset LIMIT :size ", parameters, VIEW_ROW_MAPPER);
    }

    @Override
    public Order save(Order order) {
        BeanPropertySqlParameterSource parameterSource = new BeanPropertySqlParameterSource(order);
        parameterSource.registerSqlType("paymentMethod", Types.VARCHAR);
        parameterSource.registerSqlType("orderStatus", Types.VARCHAR);
        if (order.isNew()) {
            Number newId = insertTest.executeAndReturnKey(parameterSource);
            order.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE orders SET " +
                            "external_order_id=:externalOrderId, payment_method=:paymentMethod, amount=:amount, " +
                            "test_quantity=:testQuantity, order_status=:orderStatus, rate_id=:rateId, " +
                            "user_id=:userId, order_date=:orderDate, payment_date=:paymentDate," +
                            "documents_requested_date=:documentsRequestedDate, documents_responded_date=:documentsRespondedDate, " +
                            "documents_post_name=:documentsPostName, documents_post_address=:documentsPostAddress " +
                            " WHERE id=:id"
                    , parameterSource) == 0) {
                return null;
            }
        }
        return order;
    }

    @Override
    public Order get(long id) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM orders WHERE id = " + id, ROW_MAPPER));
    }

    @Override
    public int countAll(long userId) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM orders WHERE user_id=?", Integer.class, userId);
    }

    @Override
    public int countAll() {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM orders", Integer.class);
    }
}
