package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.repository.QuestionRepository;
import com.google.common.primitives.Longs;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.example.new_test_lab.util.AppUtil.getDirection;
import static com.example.new_test_lab.util.AppUtil.getSortColumnName;


@Repository
public class QuestionRepositoryImpl implements QuestionRepository {
    private static final RowMapper<Question> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Question.class);

    private static final String DEFAULT_SORT_BY = "id";
    private static final List<String> ALLOWED_COLUMN_NAMES = Arrays.asList("id", "description", "topic_id", "type", "level");

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public QuestionRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("questions")
                .usingGeneratedKeyColumns("id");
    }

    @Override
    public Question get(long id) {
        return DataAccessUtils.singleResult(jdbcTemplate.query("SELECT * FROM questions WHERE id =?", ROW_MAPPER, id));
    }

    @Override
    public boolean delete(long id) {
        return jdbcTemplate.update("DELETE FROM questions WHERE id=?", id) != 0;
    }

    @Override
    public Question save(Question question) {
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(question);
        source.registerSqlType("level", Types.VARCHAR);
        source.registerSqlType("type", Types.VARCHAR);

        if (question.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            question.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE questions " +
                            "   SET description=:description, topic_id=:topicId, level=:level, increased_complexity=:increasedComplexity, type=:type" +
                            " WHERE id=:id"
                    , source) == 0) {
                return null;
            }
        }
        return question;
    }

    @Override
    public List<Question> getAllByLevelAndTopicIds(Level level, long... topicIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("topicIds", Longs.asList(topicIds));
        parameters.addValue("level", level.getCode());
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM questions WHERE topic_id in (:topicIds) AND level =:level", parameters, ROW_MAPPER);
    }

    @Override
    public List<Question> getAllByComplexityAndTopicIds(boolean increasedComplexity, long... topicIds) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("topicIds", Longs.asList(topicIds));
        parameters.addValue("increasedComplexity", increasedComplexity);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM questions WHERE topic_id in (:topicIds) AND increased_complexity =:increasedComplexity", parameters, ROW_MAPPER);
    }

    @Override
    public List<Question> getAllByTestId(long testId) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("testId", testId);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM questions q INNER JOIN test_questions tq ON tq.question_id=q.id WHERE tq.test_id=:testId ORDER BY q.topic_id", parameters, ROW_MAPPER);
    }

    @Override
    public List<Question> getAllByTopic(long topicId, int page, int size, String sort) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("topicId", topicId);
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        parameters.addValue("sort", sortBy);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM questions WHERE topic_id=:topicId ORDER BY " + sortBy + " OFFSET :offset LIMIT :size", parameters, ROW_MAPPER);
    }

    @Override
    public int countAllByTopicId(long topicId) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM questions WHERE topic_id=?", Integer.class, topicId);
    }

    @Override
    public int countAllByLevelAndTopicIds(Level level, long... topicId) {
        Map<String, Object> params = new HashMap<>();
        params.put("ids", Longs.asList(topicId));
        params.put("level", level.getCode());
        SqlParameterSource parameters = new MapSqlParameterSource(params);
        return namedParameterJdbcTemplate.queryForObject("SELECT count(1) FROM questions WHERE topic_id in (:ids) AND level=:level", parameters, Integer.class);
    }

    @Override
    public int countAllByComplexityAndTopicIds(boolean increasedComplexity, long... topicId) {
        Map<String, Object> params = new HashMap<>();
        params.put("ids", Longs.asList(topicId));
        params.put("increasedComplexity", increasedComplexity);
        SqlParameterSource parameters = new MapSqlParameterSource(params);
        return namedParameterJdbcTemplate.queryForObject("SELECT count(1) FROM questions WHERE topic_id in (:ids) AND increased_complexity=:increasedComplexity", parameters, Integer.class);
    }

    @Override
    public int countAllByTopicIds(long... topicId) {
        Map<String, Object> params = new HashMap<>();
        params.put("ids", Longs.asList(topicId));
        SqlParameterSource parameters = new MapSqlParameterSource(params);
        return namedParameterJdbcTemplate.queryForObject("SELECT count(1) FROM questions WHERE topic_id in (:ids)", parameters, Integer.class);
    }

    @Override
    public int[] saveTestQuestions(long id, long... questionIds) {
        return jdbcTemplate.batchUpdate("INSERT INTO test_questions(test_id, question_id) VALUES (?, ?)",
                new BatchPreparedStatementSetter() {
                    @Override
                    public void setValues(PreparedStatement statement, int i) throws SQLException {
                        statement.setLong(1, id);
                        statement.setLong(2, questionIds[i]);
                    }

                    @Override
                    public int getBatchSize() {
                        return questionIds.length;
                    }
                });
    }
}
