package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.Test;
import com.example.new_test_lab.repository.TestRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.Arrays;
import java.util.List;

import static com.example.new_test_lab.util.AppUtil.*;


@Repository
public class TestRepositoryImpl implements TestRepository {
    private static final RowMapper<Test> ROW_MAPPER = BeanPropertyRowMapper.newInstance(Test.class);
    private static final String DEFAULT_SORT_BY = "created";
    private static final List<String> ALLOWED_COLUMN_NAMES = Arrays.asList("id", "name", "description", "duration", "created", "level", "position");

    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertTest;

    public TestRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.insertTest = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("tests")
                .usingGeneratedKeyColumns("id");
        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public List<Test> getAll(long userId, int page, int size, String sort) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userId", userId);
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);
        return namedParameterJdbcTemplate.query(
                "SELECT * FROM tests WHERE user_id=:userId ORDER BY " + sortBy + " OFFSET :offset LIMIT :size ", parameters, ROW_MAPPER);
    }

    @Override
    public List<Test> getAll(long userId, int page, int size, String sort, String levels, String positions) {
        int offset = page * size;
        String sortBy = getSortColumnName(sort, ALLOWED_COLUMN_NAMES, DEFAULT_SORT_BY) + " " + getDirection(sort);
        List<String> levelFilter = getLevelsFilter(levels);
        List<String> positionFilter = getPositionsFilter(positions);

        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("userId", userId);
        parameters.addValue("offset", offset);
        parameters.addValue("size", size);

        StringBuilder queryBuilder = new StringBuilder();
        queryBuilder.append("SELECT * FROM tests WHERE user_id=:userId");
        if (levelFilter != null && !levelFilter.isEmpty()) {
            parameters.addValue("levels", levelFilter);
            queryBuilder.append(" AND level IN (:levels)");
        }
        if (positionFilter != null && !positionFilter.isEmpty()) {
            parameters.addValue("positions", positionFilter);
            queryBuilder.append(" AND position IN (:positions)");
        }
        queryBuilder.append(" ORDER BY ").append(sortBy).append(" OFFSET :offset LIMIT :size");

        return namedParameterJdbcTemplate.query(queryBuilder.toString(), parameters, ROW_MAPPER);
    }

    @Override
    public Test save(Test test, long userId) {
        test.setUserId(userId);
        BeanPropertySqlParameterSource source = new BeanPropertySqlParameterSource(test);
        if (test.isNew()) {
            Number newId = insertTest.executeAndReturnKey(source);
            test.setId(newId.longValue());
        } else {
            if (namedParameterJdbcTemplate.update("" +
                            "UPDATE tests " +
                            "   SET description=:description, name=:name, duration=:duration, increased_complexity=:increasedComplexity " +
                            " WHERE id=:id AND user_id=:userId"
                    , source) == 0) {
                return null;
            }
        }
        return test;
    }

    @Override
    public boolean delete(long id, long userId) {
        return jdbcTemplate.update("DELETE FROM tests WHERE id=? AND user_id=?", id, userId) != 0;
    }

    @Override
    public int countAll(long userId) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM tests WHERE user_id=?", Integer.class, userId);
    }

    @Override
    public int countAll(long userId, String levels, String positions) {
        MapSqlParameterSource parameters = new MapSqlParameterSource("userId", userId);
        List<String> levelFilter = getLevelsFilter(levels);
        List<String> positionFilter = getPositionsFilter(positions);

        StringBuilder queryBuilder = new StringBuilder();
        queryBuilder.append("SELECT count(1) FROM tests WHERE user_id=:userId");
        if (levelFilter != null && !levelFilter.isEmpty()) {
            parameters.addValue("levels", levelFilter);
            queryBuilder.append(" AND level IN (:levels)");
        }
        if (positionFilter != null && !positionFilter.isEmpty()) {
            parameters.addValue("positions", positionFilter);
            queryBuilder.append(" AND position IN (:positions)");
        }

        return namedParameterJdbcTemplate.queryForObject(queryBuilder.toString(), parameters, Integer.class);
    }

    @Override
    public Test get(long id, long userId) {
        MapSqlParameterSource parameters = new MapSqlParameterSource();
        parameters.addValue("id", id);
        parameters.addValue("userId", userId);

        return DataAccessUtils.singleResult(namedParameterJdbcTemplate.query(
                "SELECT * FROM tests WHERE id =:id AND user_id=:userId", parameters, ROW_MAPPER));
    }
}
