package com.example.new_test_lab.repository.jdbc_impl;

import com.example.new_test_lab.model.User;
import com.example.new_test_lab.model.enums.Role;
import com.example.new_test_lab.repository.UserRepository;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;


import java.util.*;

@Repository
public class UserRepositoryImpl implements UserRepository {
    private static final BeanPropertyRowMapper<User> ROW_MAPPER = BeanPropertyRowMapper.newInstance(User.class);
    private final JdbcTemplate jdbcTemplate;
    private final NamedParameterJdbcTemplate namedParameterJdbcTemplate;
    private final SimpleJdbcInsert insertUser;

    public UserRepositoryImpl(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        this.insertUser = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("users")
                .usingGeneratedKeyColumns("id");

        this.jdbcTemplate = jdbcTemplate;
        this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

    @Override
    public User save(User user, long userId) {
        BeanPropertySqlParameterSource parameterSource = new BeanPropertySqlParameterSource(user);
        if (user.isNew()) {
            Number newKey = insertUser.executeAndReturnKey(parameterSource);
            user.setId(newKey.longValue());
            insertRoles(user);
        } else {
            if (namedParameterJdbcTemplate.update(
                    "UPDATE users SET email=:email, phone=:phone, password=:password, " +
                            "confirmed=:confirmed, confirmation_code=:confirmationCode, reset_password_code=:resetPasswordCode, " +
                            "enabled=:enabled, first_name=:firstName, last_name=:lastName, middle_name=:middleName, company_info_id=:companyInfoId " +
                            "WHERE id=:id AND (id=" + userId + "OR parent_id= " + userId + ")", parameterSource) == 0) {
                return null;
            }
            deleteRoles(user);
            insertRoles(user);
        }
        return user;
    }

    @Override
    public User get(long id, long userId) {
        List<User> users = jdbcTemplate.query("SELECT * FROM users WHERE id=? AND (id=? OR parent_id=?)", ROW_MAPPER, id, userId, userId);
        return setRoles(DataAccessUtils.singleResult(users));
    }

    @Override
    public User getByResetPasswordCode(String resetPasswordCode) {
        List<User> users = jdbcTemplate.query("SELECT * FROM users WHERE reset_password_code=?", ROW_MAPPER, resetPasswordCode);
        return setRoles(DataAccessUtils.singleResult(users));
    }

    @Override
    public List<User> getAll(long userId) {
        Map<Long, Set<Role>> map = new HashMap<>();
        jdbcTemplate.query("SELECT * FROM user_roles", rs -> {
            map.computeIfAbsent(rs.getLong("user_id"), id -> EnumSet.noneOf(Role.class))
                    .add(Role.valueOf(rs.getString("role")));
        });
        List<User> users = jdbcTemplate.query("SELECT * FROM users WHERE parent_id=? ORDER BY email", ROW_MAPPER, userId);
        users.forEach(u -> u.setRoles(map.get(u.getId())));
        return users;
    }

    @Override
    public int countAll(long userId) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM users WHERE parent_id = ?", Integer.class, userId);
    }

    @Override
    public boolean delete(long id, long userId) {
        return jdbcTemplate.update("DELETE FROM users WHERE id=? AND parent_id=?", id, userId) != 0;
    }

    @Override
    public User getByEmail(String email) {
        List<User> users = jdbcTemplate.query("SELECT * FROM users WHERE email=?", ROW_MAPPER, email);
        return setRoles(DataAccessUtils.singleResult(users));
    }

    @Override
    public boolean hasRole(long userId, Role role) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM user_roles WHERE user_id = ? AND role = ?", Integer.class, userId, role.name()) > 0;
    }

    @Override
    public boolean isEmailExists(String email) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM users WHERE email = ?", Integer.class, email) > 0;
    }

    @Override
    public boolean isPhoneExists(String phone) {
        return jdbcTemplate.queryForObject("SELECT count(1) FROM users WHERE phone = ?", Integer.class, phone) > 0;
    }

    private void insertRoles(User u) {
        Set<Role> roles = u.getRoles();
        if (!CollectionUtils.isEmpty(roles)) {
            jdbcTemplate.batchUpdate("INSERT INTO user_roles (user_id, role) VALUES (?, ?)", roles, roles.size(),
                    (ps, role) -> {
                        ps.setLong(1, u.getId());
                        ps.setString(2, role.name());
                    });
        }
    }

    private void deleteRoles(User u) {
        jdbcTemplate.update("DELETE FROM user_roles WHERE user_id=?", u.getId());
    }

    private User setRoles(User u) {
        if (u != null) {
            Set<Role> roles = EnumSet.copyOf(jdbcTemplate.queryForList("SELECT role FROM user_roles  WHERE user_id=?", Role.class, u.getId()));
            u.setRoles(roles);
        }
        return u;
    }
}
