package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.model.ResultCode;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.repository.CandidateAnswerRepository;
import com.example.new_test_lab.repository.CandidateQuestionRepository;
import com.example.new_test_lab.repository.CandidateTestRepository;
import com.example.new_test_lab.repository.ResultCodeRepository;
import com.example.new_test_lab.service.CandidateQuestionService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static com.example.new_test_lab.util.ValidationUtil.checkNotFound;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
@AllArgsConstructor
public class CandidateQuestionServiceImpl implements CandidateQuestionService {

    private CandidateTestRepository candidateTestRepository;
    private CandidateQuestionRepository candidateQuestionRepository;
    private CandidateAnswerRepository candidateAnswerRepository;
    private ResultCodeRepository resultCodeRepository;

    @Override
    public List<CandidateAnswer> getAnswers(long... candidateQuestionIds) {
        return candidateAnswerRepository.getByQuestionId(candidateQuestionIds);
    }

    @Override
    @Transactional
    public void answer(String linkCode, Long questionId, List<Long> answerIds) {
        log.info("Putting answer for question with id={} test with link code {}", questionId, linkCode);

        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getByCode(linkCode), "link code=" + linkCode);
        checkInProgress(candidateTest);
        CandidateQuestion question = checkNotFoundWithId(candidateQuestionRepository.get(candidateTest.getId(), questionId), questionId);
        List<CandidateAnswer> answers = new ArrayList<>();
        if (question.isAnswered())
            throw new ServerException(Exceptions.QUESTIONS_ALREADY_ANSWERED);
        question.setAnswered(true);
        answerIds.forEach(id -> {
            CandidateAnswer answer = checkNotFoundWithId(candidateAnswerRepository.get(id, questionId), id);
            answer.setSelected(true);
            answers.add(answer);
        });
        candidateQuestionRepository.save(question);
        candidateAnswerRepository.saveAll(answers);
    }

    @Override
    public List<CandidateQuestion> getQuestionsByTestId(long candidateTestId, int page, int size) {
        log.info("Getting questions by test id={}", candidateTestId);

        return candidateQuestionRepository.getByTestId(candidateTestId, page, size);
    }

    @Override
    public List<CandidateQuestion> getQuestionsByLinkCode(String linkCode, int page, int size) {
        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getByCode(linkCode), "link code=" + linkCode);
        checkInProgress(candidateTest);

        return candidateQuestionRepository.getByTestId(candidateTest.getId(), page, size);
    }

    // Doesn't throw because we check result for null in TestResultController
    @Override
    public List<CandidateQuestion> getAllQuestionsByResultCode(String resultCode) {
        ResultCode code = checkNotFound(resultCodeRepository.findByCode(resultCode), "result code=" + resultCode);
        if (!code.isCanViewAnswers())
            return null;

        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getById(code.getCandidateTestId()),
                                                    "id=" + code.getCandidateTestId());
        checkFinished(candidateTest);

        List<CandidateQuestion> result = candidateQuestionRepository.getByTestId(candidateTest.getId());
        // To avoid returning null if code.isCanViewAnswers() == true
        return result == null ? Collections.emptyList() : result;
    }

    @Override
    public List<CandidateQuestion> getQuestionsByResultCode(String resultCode, int page, int size) {
        ResultCode code = checkNotFound(resultCodeRepository.findByCode(resultCode), "result code=" + resultCode);
        if (!code.isCanViewAnswers())
            throw new ServerException(Exceptions.CAN_NOT_VIEW_ANSWERS);

        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getById(code.getCandidateTestId()),
                                                    "id=" + code.getCandidateTestId());
        checkFinished(candidateTest);

        return candidateQuestionRepository.getByTestId(candidateTest.getId(), page, size);
    }

    private void checkInProgress(CandidateTest candidateTest) {
        if (candidateTest.getStatus() != CandidateTestStatus.IN_PROGRESS)
            throw new ServerException(Exceptions.WRONG_STATUS);
    }

    private void checkFinished(CandidateTest candidateTest) {
        if (candidateTest.getStatus() != CandidateTestStatus.FINISHED)
            throw new ServerException(Exceptions.WRONG_STATUS);
    }
}
