package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.dto.TopicStatDto;
import com.example.new_test_lab.email.EmailService;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.*;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.model.enums.Role;
import com.example.new_test_lab.repository.*;
import com.example.new_test_lab.service.CandidateTestService;
import com.example.new_test_lab.service.UserService;
import com.example.new_test_lab.util.AppUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.createCandidateAnswers;
import static com.example.new_test_lab.util.AppUtil.createCandidateQuestion;
import static com.example.new_test_lab.util.TestUtil.fillNewCandidateTest;
import static com.example.new_test_lab.util.ValidationUtil.*;
import static org.springframework.transaction.annotation.Propagation.NEVER;


@Slf4j
@Service
@Transactional(readOnly = true)
public class CandidateTestServiceImpl implements CandidateTestService {

    private static final int SCHEDULER_DELAY = 5000;

    private TestRepository testRepository;
    private CandidateTestRepository candidateTestRepository;
    private QuestionRepository questionRepository;
    private CandidateQuestionRepository candidateQuestionRepository;
    private AnswerRepository answerRepository;
    private TopicRepository topicRepository;
    private CandidateAnswerRepository candidateAnswerRepository;
    private AccountRepository accountRepository;
    private UserRepository userRepository;
    private UserService userService;
    private ResultCodeRepository resultCodeRepository;

    @Lazy
    @Autowired
    private EmailService emailService;

    @Autowired
    public CandidateTestServiceImpl(TestRepository testRepository,
                                    CandidateTestRepository candidateTestRepository,
                                    QuestionRepository questionRepository,
                                    CandidateQuestionRepository candidateQuestionRepository,
                                    AnswerRepository answerRepository,
                                    TopicRepository topicRepository,
                                    CandidateAnswerRepository candidateAnswerRepository,
                                    AccountRepository accountRepository,
                                    UserService userService,
                                    UserRepository userRepository,
                                    ResultCodeRepository resultCodeRepository) {
        this.testRepository = testRepository;
        this.candidateTestRepository = candidateTestRepository;
        this.questionRepository = questionRepository;
        this.candidateQuestionRepository = candidateQuestionRepository;
        this.answerRepository = answerRepository;
        this.topicRepository = topicRepository;
        this.candidateAnswerRepository = candidateAnswerRepository;
        this.accountRepository = accountRepository;
        this.userService = userService;
        this.userRepository = userRepository;
        this.resultCodeRepository = resultCodeRepository;
    }

    @Override
    @Transactional
    public CandidateTest create(CandidateTest candidateTest, long userId) {
        log.info("Creating candidate test {} for user with id={}", candidateTest, userId);

        checkLimits(userId);
        checkNew(candidateTest, true);
        Test test = checkNotFoundWithId(testRepository.get(candidateTest.getTestId(), userId), candidateTest.getTestId());
        return addNewCandidateTest(test, candidateTest);
    }

    @Override
    @Transactional
    public void delete(long id, long userId) {
        log.info("Deleting candidate test with id={}", id);

        checkNotFoundWithId(candidateTestRepository.delete(id, userId), id);
    }

    @Override
    @Transactional
    public void archive(long id, long userId) {
        log.info("Archive candidate test with id={}", id);
        CandidateTest candidateTest = checkNotFoundWithId(candidateTestRepository.getByIdAndUserIds(id, userId), id);
        checkNotArchived(candidateTest);

        candidateTestRepository.archive(id, userId);
    }

    @Override
    @Transactional
    public void unarchive(long id, long userId) {
        log.info("Unarchive candidate test with id={}", id);
        CandidateTest candidateTest = checkNotFoundWithId(candidateTestRepository.getByIdAndUserIds(id, userId), id);
        checkArchived(candidateTest);

        candidateTestRepository.unarchive(id, userId);
    }

    @Override
    public List<TopicStatDto> getTopicStatistic(long id) {
        log.info("Getting statistics for topic with id={}", id);

        return getCorrectQuestionNumByTopic(id);
    }

    @Override
    @Transactional
    public CandidateTest start(String linkCode) {
        log.info("Starting test with linkCode={}", linkCode);

        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getByCode(linkCode), "link code=" + linkCode);
        if (candidateTest.getStatus() != CandidateTestStatus.NEW)
            throw new ServerException(Exceptions.CANT_START_TEST);
        candidateTest.setStatus(CandidateTestStatus.IN_PROGRESS);
        candidateTest.setTestStarted(LocalDateTime.now());
        userService.decreaseTestBalance(candidateTest.getUserId());
        return candidateTestRepository.save(candidateTest, candidateTest.getUserId());
    }

    @Override
    @Transactional
    public CandidateTest finish(String linkCode) {
        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getByCode(linkCode), "link code=" + linkCode);
        if (candidateTest.getStatus() != CandidateTestStatus.IN_PROGRESS)
            throw new ServerException(Exceptions.CANT_FINISH_TEST);
        return finishTest(candidateTest);
    }

    @Override
    public CandidateTest getByLinkCode(String linkCode) {
        return checkNotFound(candidateTestRepository.getByCode(linkCode), "link code=" + linkCode);
    }

    @Override
    public CandidateTest getByResultCode(String resultCode) {
        ResultCode code = checkNotFound(resultCodeRepository.findByCode(resultCode), "result code=" + resultCode);
        CandidateTest candidateTest = checkNotFound(candidateTestRepository.getById(code.getCandidateTestId()),
                                                    "id=" + code.getCandidateTestId());

        checkFinished(candidateTest);

        return candidateTest;
    }

    @Override
    public List<CandidateTest> getAll(long userId, int page, int size, String sort) {
        return candidateTestRepository.getAll(page, size, sort, userId);
    }

    @Override
    public List<CandidateTest> getAll(long userId, int page, int size, String sort, String level, String position, String status, Boolean isArchived) {
        return candidateTestRepository.getAll(page, size, sort, level, position, status, isArchived, userId);
    }

    @Override
    public int countAll(long userId) {
        return candidateTestRepository.countAll(userId);
    }

    @Override
    public int countAll(long userId, String level, String position, String status, Boolean isArchived) {
        return candidateTestRepository.countAll(level, position, status, isArchived, userId);
    }

    @Override
    public CandidateTest get(long id, long userId) {
        return checkNotFound(candidateTestRepository.getByIdAndUserIds(id, userId), "id=" + id);
    }

    @Override
    @Transactional
    public void setViewed(CandidateTest candidateTest, long userId) {
        candidateTest.setNotViewed(false);
        candidateTestRepository.save(candidateTest, userId);
    }

    private CandidateTest addNewCandidateTest(Test test, CandidateTest candidateTest) {
        fillNewCandidateTest(test, candidateTest);
        candidateTest = candidateTestRepository.save(candidateTest, test.getUserId());
        List<Question> questions = questionRepository.getAllByTestId(test.getId());
        List<Topic> topics = topicRepository.getByIds(questions.stream().mapToLong(Question::getTopicId).distinct().toArray());
        Map<Long, List<Answer>> answerMap = answerRepository.getByTestId(test.getId()).stream().collect(Collectors.groupingBy(Answer::getQuestionId));
        long candidateTestId = candidateTest.getId();
        Map<Long, String> topicNames = topics.stream().collect(Collectors.toMap(Topic::getId, Topic::getName));
        List<CandidateAnswer> answers = new ArrayList<>();
        questions.forEach(q -> {
            CandidateQuestion created = candidateQuestionRepository.save(createCandidateQuestion(q, candidateTestId, topicNames.get(q.getTopicId())));
            answers.addAll(createCandidateAnswers(answerMap.get(q.getId()), created.getId()));
        });
        candidateAnswerRepository.saveAll(answers);
        return candidateTest;
    }

    @Async
    @Scheduled(fixedDelay = SCHEDULER_DELAY)
    @Transactional(propagation = NEVER)
    public void checkExpiredTests() {
        List<CandidateTest> candidateTests = candidateTestRepository.getExpired();
        candidateTests.forEach(this::finishTest);
    }

    private CandidateTest finishTest(CandidateTest candidateTest) {
        candidateTest.setStatus(CandidateTestStatus.FINISHED);
        candidateTest.setTestFinished(LocalDateTime.now());
        candidateTest.setCorrectQuestionNumber(getCorrectQuestionNum(candidateTest.getId()));
        candidateTest.setNotViewed(true);
        candidateTest = candidateTestRepository.save(candidateTest, candidateTest.getUserId());
        User user = userRepository.get(candidateTest.getUserId(), candidateTest.getUserId());
        emailService.sendTestFinished(user.getEmail(),
                candidateTest.getCandidateName(),
                candidateTest.getName(),
                String.valueOf(Math.round((float)(candidateTest.getCorrectQuestionNumber() / candidateTest.getQuestionNumber()) * 100)),
                String.valueOf(candidateTest.getId()));
        return candidateTest;
    }

    private Integer getCorrectQuestionNum(long id) {
        List<CandidateQuestion> questions = candidateQuestionRepository.getByTestId(id);
        Map<Long, List<CandidateAnswer>> answerMap = candidateAnswerRepository.getByQuestionId(questions.stream().mapToLong(CandidateQuestion::getId).toArray()).stream().collect(Collectors.groupingBy(CandidateAnswer::getCandidateQuestionId));
        return (int) questions.stream().filter(q -> AppUtil.isAnsweredCorrect(answerMap.get(q.getId()), q.getType())).count();
    }

    private List<TopicStatDto> getCorrectQuestionNumByTopic(long id) {
        List<CandidateQuestion> questions = candidateQuestionRepository.getByTestId(id);
        Map<Long, List<CandidateAnswer>> answerMap = candidateAnswerRepository.getByQuestionId(questions.stream().mapToLong(CandidateQuestion::getId).toArray()).stream().collect(Collectors.groupingBy(CandidateAnswer::getCandidateQuestionId));
        return questions.stream().collect(Collectors.groupingBy(CandidateQuestion::getTopicName))
                .entrySet().stream().map(e -> new TopicStatDto(e.getKey(), e.getValue().size(), (int) e.getValue().stream().filter(q -> AppUtil.isAnsweredCorrect(answerMap.get(q.getId()), q.getType())).count())).collect(Collectors.toList());
    }

    private void checkLimits(long userId) {
        if (userId > 0 && userRepository.hasRole(userId, Role.ROLE_ADMIN)) {
            Account account = accountRepository.get(userId);
            if (account.getTestBalance() < 1)
                throw new ServerException(Exceptions.CAN_NOT_CREATE_MORE_TESTS);
        }
    }

    private void checkFinished(CandidateTest candidateTest) {
        checkStatus(candidateTest, CandidateTestStatus.FINISHED);
    }

    private void checkArchived(CandidateTest candidateTest) {
        if (!candidateTest.isArchived()) throw new ServerException(Exceptions.WRONG_STATUS);
    }

    private void checkNotArchived(CandidateTest candidateTest) {
        if (candidateTest.isArchived()) throw new ServerException(Exceptions.WRONG_STATUS);
    }

    private void checkStatus(CandidateTest candidateTest, CandidateTestStatus status) {
        if (candidateTest.getStatus() != status)
            throw new ServerException(Exceptions.WRONG_STATUS);
    }
}
