package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.dto.DocumentsRequestDto;
import com.example.new_test_lab.dto.OrderDto;
import com.example.new_test_lab.event.OrderStatusChangedEvent;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.Account;
import com.example.new_test_lab.model.CompanyInfo;
import com.example.new_test_lab.model.Order;
import com.example.new_test_lab.model.OrderView;
import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.model.enums.OrderStatus;
import com.example.new_test_lab.model.enums.PaymentMethod;
import com.example.new_test_lab.model.enums.Role;
import com.example.new_test_lab.repository.AccountRepository;
import com.example.new_test_lab.repository.OrderRepository;
import com.example.new_test_lab.repository.RateRepository;
import com.example.new_test_lab.repository.SysParamsRepository;
import com.example.new_test_lab.repository.UserRepository;
import com.example.new_test_lab.service.CompanyInfoService;
import com.example.new_test_lab.service.OrderService;
import com.example.new_test_lab.tinkoff.Tinkoff;
import com.example.new_test_lab.tinkoff.model.Payment;
import com.example.new_test_lab.util.docx.DocxReplacer;
import com.fasterxml.jackson.databind.exc.InvalidFormatException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;

import static com.example.new_test_lab.exception.Exceptions.NOT_FOUND;
import static com.example.new_test_lab.util.OrderUtil.createNewOrder;
import static com.example.new_test_lab.util.OrderUtil.getAmountInSmallestUnit;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;
import static com.example.new_test_lab.util.docx.DocxConstants.DOCUMENT_DATE_PATTERN;
import static com.example.new_test_lab.util.docx.DocxConstants.DOCUMENT_NAME_DATE_PATTERN;
import static com.example.new_test_lab.util.docx.DocxConstants.DOCUMENT_PRICE_PATTERN;
import static com.example.new_test_lab.util.docx.DocxConstants.INVOICE_VALID_FOR_2_WEEKS;
import static com.example.new_test_lab.util.docx.DocxConstants.NDS_PERCENT;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_BILL_PRICE;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_ID;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_INN;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_KPP;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_LEGAL_NAME_COMP;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_NDS_PRICE;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_ORDER_DATE;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_PAYMENT_UNTIL_DATE;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_POST_ADDRESS;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_RATE_NAME;
import static com.example.new_test_lab.util.docx.DocxConstants.PLACEHOLDER_TYPE_COMP;


@Slf4j
@Service
@Transactional(readOnly = true)
public class OrderServiceImpl implements OrderService {

    private OrderRepository orderRepository;
    private RateRepository rateRepository;
    private AccountRepository accountRepository;
    private UserRepository userRepository;
    private CompanyInfoService companyInfoService;
    private SysParamsRepository sysParamsRepository;

    @Lazy
    @Autowired
    private Tinkoff tinkoff;

    public OrderServiceImpl(OrderRepository orderRepository,
                            RateRepository rateRepository,
                            AccountRepository accountRepository,
                            UserRepository userRepository,
                            SysParamsRepository sysParamsRepository,
                            CompanyInfoService companyInfoService) {
        this.orderRepository = orderRepository;
        this.rateRepository = rateRepository;
        this.accountRepository = accountRepository;
        this.userRepository = userRepository;
        this.sysParamsRepository = sysParamsRepository;
        this.companyInfoService = companyInfoService;
    }

    @Override
    @Transactional
    public OrderDto createOrder(long rateId, long userId, PaymentMethod paymentMethod) {
        Rate rate = checkNotFoundWithId(rateRepository.get(rateId), rateId);
        Order order = createNewOrder(rate, userId, paymentMethod);
        orderRepository.save(order);
        if (PaymentMethod.BANK_CARD.getCode().equals(paymentMethod.getCode())) {
            Payment payment = tinkoff.initPayment(order.getId(), getAmountInSmallestUnit(order.getAmount()));
            order.setExternalOrderId(payment.getPaymentId());
        }
        return new OrderDto(order.getId(), null /*payment.getPaymentURL()*/);
    }

    @Override
    public Order get(long id, long userId) {
        Order order = checkNotFoundWithId(orderRepository.get(id), id);
        if (userRepository.hasRole(userId, Role.ROLE_SUPERUSER)) {
            return order;
        } else {
            return checkNotFoundWithId(order.getUserId() == userId ? order : null, id);
        }
    }

    @Override
    public List<OrderView> getAll(long userId, int page, int size, String sort) {
        if (userRepository.hasRole(userId, Role.ROLE_SUPERUSER)) {
            return orderRepository.getAll(page, size, sort);
        } else {
            return orderRepository.getAll(userId, page, size, sort);
        }
    }

    @Override
    public int countAll(long userId) {
        if (userRepository.hasRole(userId, Role.ROLE_SUPERUSER)) {
            return orderRepository.countAll();
        } else {
            return orderRepository.countAll(userId);
        }
    }

    @Transactional
    @EventListener
    public void onOrderStatusChanged(OrderStatusChangedEvent event) {
        Order order = orderRepository.get(event.getOrderId());
        if (order == null) {
            log.error("Order with order id {} not found", event.getOrderId());
            return;
        }

        if (event.getStatus() == OrderStatus.CONFIRMED && order.getOrderStatus() != OrderStatus.CONFIRMED) {
            Account account = accountRepository.get(order.getUserId());
            account.setTestBalance(order.getTestQuantity());
            account.setSubscriptionStart(LocalDateTime.now());
            account.setSubscriptionEnd(LocalDateTime.now().plusDays(order.getSubscriptionDays()));
            accountRepository.save(account);
        }
        order.setOrderStatus(event.getStatus());
        orderRepository.save(order);
    }

    @Override
    @Transactional
    public void requestDocuments(DocumentsRequestDto documentsRequest, long userId) {
        Order order = get(documentsRequest.getOrderId(), userId);

        if (order == null) {
            throw new ServerException(NOT_FOUND);
        }

        order.setDocumentsPostName(documentsRequest.getDocumentsPostName());
        order.setDocumentsPostAddress(documentsRequest.getDocumentsPostAddress());
        order.setDocumentsRequestedDate(LocalDateTime.now());
        orderRepository.save(order);
    }

    @Override
    @Transactional
    public String downloadInvoice(long orderId, long companyInfoId) {
        if (companyInfoId == 0) throw new ServerException(Exceptions.FILL_COMPANY_INFO);
        Order order = orderRepository.get(orderId);
        if (order == null) {
            log.error("Order with order id {} not found", orderId);
            throw new ServerException(Exceptions.BAD_REQUEST);
        }
        CompanyInfo companyInfo = companyInfoService.findCompanyInfoById(companyInfoId);
        if (companyInfo == null) {
            log.error("Company Info with order id {} not found", companyInfoId);
            throw new ServerException(Exceptions.BAD_REQUEST);
        }
        Rate rate = rateRepository.get(order.getRateId());
        if (rate == null) {
            log.error("Rate with order id {} not found", order.getRateId());
            throw new ServerException(Exceptions.BAD_REQUEST);
        }
        String resultFile = String.format(sysParamsRepository.get().getTempInvoiceStorePath(),   //"\\opt\\temp\\invoices\\order-%d-%s"
                order.getId(),
                order.getOrderDate().format(DateTimeFormatter.ofPattern(DOCUMENT_NAME_DATE_PATTERN)));
        try {
            DocxReplacer docxView = new DocxReplacer(getClass().getClassLoader().getResourceAsStream("order.docx"));

            docxView.replace(new HashMap<String, String>() {{
                put(PLACEHOLDER_ID, order.getId().toString());
                put(PLACEHOLDER_ORDER_DATE, order.getOrderDate().format(DateTimeFormatter.ofPattern(DOCUMENT_DATE_PATTERN)));
                put(PLACEHOLDER_TYPE_COMP, companyInfo.getTypeComp());
                put(PLACEHOLDER_LEGAL_NAME_COMP, companyInfo.getLegalNameComp());
                put(PLACEHOLDER_INN, companyInfo.getInn());
                put(PLACEHOLDER_KPP, companyInfo.getKpp());
                put(PLACEHOLDER_POST_ADDRESS, companyInfo.getPostAddress());
                put(PLACEHOLDER_RATE_NAME, rate.getName());
                put(PLACEHOLDER_BILL_PRICE, new DecimalFormat(DOCUMENT_PRICE_PATTERN).format(rate.getBillPrice()));
                put(PLACEHOLDER_NDS_PRICE, new DecimalFormat(DOCUMENT_PRICE_PATTERN).format(rate.getBillPrice() * NDS_PERCENT));
                put(PLACEHOLDER_PAYMENT_UNTIL_DATE, order.getOrderDate().plusDays(INVOICE_VALID_FOR_2_WEEKS).format(DateTimeFormatter.ofPattern(DOCUMENT_DATE_PATTERN)));
            }});

            docxView.writeAndClose(resultFile);
            /*docxView.convertToPdf(resultFile, resultFile+PDF_FILE_EXTENSION);*/
        } catch (InvalidFormatException ex) {
            ex.printStackTrace();
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (org.apache.poi.openxml4j.exceptions.InvalidFormatException e) {
            throw new RuntimeException(e);
        }
        if (Files.exists(Paths.get(resultFile))) {
            return resultFile;
        } else {
            log.error("Can't create invoice file {}", orderId);
            throw new ServerException(Exceptions.BAD_REQUEST);
        }
    }
}
