package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.model.Answer;
import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.repository.AnswerRepository;
import com.example.new_test_lab.repository.QuestionRepository;
import com.example.new_test_lab.service.QuestionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;

import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
public class QuestionServiceImpl implements QuestionService {

    private QuestionRepository questionRepository;
    private AnswerRepository answerRepository;

    @Autowired
    public QuestionServiceImpl(QuestionRepository questionRepository, AnswerRepository answerRepository) {
        this.questionRepository = questionRepository;
        this.answerRepository = answerRepository;
    }

    @Override
    public List<Question> getAllByTestId(long testId) {
        log.info("Getting all for test id={}", testId);

        return questionRepository.getAllByTestId(testId);
    }

    @Override
    public List<Question> getAllByTopicId(long topicId, int page, int size, String sort) {
        log.info("Getting all for topic id={}", topicId);

        return questionRepository.getAllByTopic(topicId, page, size, sort);
    }

    @Override
    public int countAllByTopicId(long topicId) {
        log.info("Count all for topic id={}", topicId);

        return questionRepository.countAllByTopicId(topicId);
    }

    @Override
    public List<Answer> getAnswers(long questionId) {
        log.info("Getting answers for question with id={}", questionId);

        return answerRepository.getByQuestionId(questionId);
    }

    @Override
    public Question get(long id) {
        log.info("Getting question with id={}", id);

        return checkNotFoundWithId(questionRepository.get(id), id);
    }

    @Override
    @Transactional
    public Question create(Question question, List<Answer> answers) {
        log.info("Creating question: {}, answers: {}", question, answers);

        checkNew(question, true);
        Question created = questionRepository.save(question);
        answers.forEach(a -> {
            a.setQuestionId(created.getId());
            answerRepository.save(a);
        });
        return created;
    }

    @Override
    @Transactional
    public void update(Question question, List<Answer> answers) {
        log.info("Updating question: {}, answers: {}", question, answers);

        checkNew(question, false);
        answerRepository.deleteByQuestionId(question.getId());
        answers.forEach(a -> {
            a.setQuestionId(question.getId());
            answerRepository.save(a);
        });
        checkNotFoundWithId(questionRepository.save(question), question.getId());
    }

    @Override
    @Transactional
    public void delete(long id)
    {
        log.info("Deleting question with id={}", id);

        checkNotFoundWithId(questionRepository.delete(id), id);
    }
}
