package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.dto.CreateResultCodeDto;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.model.ResultCode;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.repository.CandidateTestRepository;
import com.example.new_test_lab.repository.ResultCodeRepository;
import com.example.new_test_lab.service.ResultCodeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.getRandomCode;
import static org.springframework.transaction.annotation.Propagation.NEVER;


@Slf4j
@Service
@RequiredArgsConstructor
public class ResultCodeServiceImpl implements ResultCodeService {

    private static final int SCHEDULER_DELAY = 60000; // every 60 seconds

    private static final int CODE_LENGTH = 32;

//    @Value("${testing.system.result-code.ttl}")
    @Value("864000")
    private Long ttl;

    private final ResultCodeRepository resultCodeRepository;

    private final CandidateTestRepository candidateTestRepository;

    @Override
    public ResultCode create(CreateResultCodeDto resultCodeDto) {
        CandidateTest candidateTest = candidateTestRepository.getById(resultCodeDto.getCandidateTestId());
        checkFinished(candidateTest);

        ResultCode resultCode = new ResultCode();
        resultCode.setCode(getRandomCode(CODE_LENGTH));
        resultCode.setCandidateTestId(resultCodeDto.getCandidateTestId());
        resultCode.setCanViewAnswers(resultCodeDto.isCanViewAnswers());
        resultCode.setTtl(ttl);
        resultCode.setCreatedAt(LocalDateTime.now());

        return resultCodeRepository.save(resultCode);
    }

    @Override
    public ResultCode getByCode(String code) {
        return resultCodeRepository.findByCode(code);
    }

    @Override
    public List<ResultCode> findAllWithCandidateTestId(Long candidateTestId, Long userId) {
        CandidateTest candidateTest = candidateTestRepository.getByIdAndUserIds(candidateTestId, userId);

        return candidateTest != null
                ? resultCodeRepository.findByCandidateTestId(candidateTestId)
                : Collections.emptyList();
    }

    @Override
    public List<ResultCode> findAll(Long userId) {
        List<CandidateTest> candidateTests = candidateTestRepository.getAll(userId);

        return candidateTests != null
                ? resultCodeRepository.findByCandidateTestIdIn(candidateTests.stream()
                                                                             .map(CandidateTest::getId)
                                                                             .collect(Collectors.toList()))
                : Collections.emptyList();
    }

    @Async
    @Scheduled(fixedDelay = SCHEDULER_DELAY)
    @Transactional(propagation = NEVER)
    public void checkExpiredResultCodes() {
        LocalDateTime now = LocalDateTime.now();
        List<ResultCode> resultCodes = resultCodeRepository.findAll();
        resultCodes.stream()
                   .filter(resultCode -> this.isExpired(resultCode, now))
                   .forEach(resultCodeRepository::delete);
    }

    private boolean isExpired(ResultCode resultCode, LocalDateTime now) {
        return now.isAfter(resultCode.getCreatedAt().plusSeconds(resultCode.getTtl()));
    }

    private void checkFinished(CandidateTest candidateTest) {
        if (candidateTest.getStatus() != CandidateTestStatus.FINISHED)
            throw new ServerException(Exceptions.WRONG_STATUS);
    }

}
