package com.example.new_test_lab.service.impl;

import com.example.new_test_lab.dto.CountQuestionsDto;
import com.example.new_test_lab.model.Test;
import com.example.new_test_lab.repository.QuestionRepository;
import com.example.new_test_lab.repository.TestRepository;
import com.example.new_test_lab.service.TestService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import java.util.List;

import static com.example.new_test_lab.util.TestUtil.pickUpQuestionsByComplexity;
import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;


@Slf4j
@Service
@Transactional(readOnly = true)
public class TestServiceImpl implements TestService {

    private TestRepository testRepository;
    private QuestionRepository questionRepository;

    @Autowired
    public TestServiceImpl(TestRepository testRepository, QuestionRepository questionRepository) {
        this.testRepository = testRepository;
        this.questionRepository = questionRepository;
    }

    @Override
    public List<Test> getAll(long userId, int page, int size, String sort) {
        log.info("Getting all for user with id={}", userId);

        return testRepository.getAll(userId, page, size, sort);
    }

    @Override
    public List<Test> getAll(long userId, int page, int size, String sort, String levels, String positions) {
        log.info("Getting all for user with id={} with filter", userId);

        return testRepository.getAll(userId, page, size, sort, levels, positions);
    }

    @Override
    public int countAll(long userId) {
        log.info("Counting all for user with id={}", userId);

        return testRepository.countAll(userId);
    }

    @Override
    public int countAll(long userId, String levels, String positions) {
        log.info("Counting all for user with id={}", userId);

        return testRepository.countAll(userId, levels, positions);
    }

    @Override
    public Test get(long id, long userId) {
        log.info("Getting test with id={} for user with id={}", id, userId);

        return checkNotFoundWithId(testRepository.get(id, userId), id);
    }

    @Override
    @Transactional
    public Test create(Test test, List<Long> topicIds, long userId) {
        log.info("Creating test {} for user with id={}", test, userId);

        checkNew(test, true);
        long[] questionIds = pickUpQuestionsByComplexity(topicIds, test.getQuestionNumber(), test.isIncreasedComplexity(), questionRepository);
        Test created = testRepository.save(test, userId);
        questionRepository.saveTestQuestions(created.getId(), questionIds);
        return created;
    }

    @Override
    @Transactional
    public void update(Test test, long userId) {
        log.info("Updating test {} for user with id={}", test, userId);

        checkNew(test, false);
        testRepository.save(test, userId);
    }

    @Override
    @Transactional
    public void delete(long id, long userId) {
        log.info("Deleting test with id={} for user with id={}", id, userId);

        checkNotFoundWithId(testRepository.delete(id, userId), id);
    }

    @Override
    @Transactional
    public CountQuestionsDto countQuestionsByTopicIds(List<Long> topicIds) {
        log.info("Question count for topics {}", topicIds);

        return new CountQuestionsDto(
                questionRepository.countAllByComplexityAndTopicIds(false, topicIds.stream().mapToLong(l -> l).toArray()),
                questionRepository.countAllByTopicIds(topicIds.stream().mapToLong(l -> l).toArray()));
    }
}
