package com.example.new_test_lab.service.impl;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.example.new_test_lab.AuthorizedUser;
import com.example.new_test_lab.email.EmailService;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.Account;
import com.example.new_test_lab.model.SysParams;
import com.example.new_test_lab.model.User;
import com.example.new_test_lab.model.enums.Role;
import com.example.new_test_lab.mutex.AccountMutexFactory;
import com.example.new_test_lab.repository.AccountRepository;
import com.example.new_test_lab.repository.SysParamsRepository;
import com.example.new_test_lab.repository.UserRepository;
import com.example.new_test_lab.service.UserService;
import com.example.new_test_lab.util.UserUtil;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static com.example.new_test_lab.util.AppUtil.getRandomCode;
import static com.example.new_test_lab.util.UserUtil.fillNewUser;
import static com.example.new_test_lab.util.ValidationUtil.checkNew;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFound;
import static com.example.new_test_lab.util.ValidationUtil.checkNotFoundWithId;

@Slf4j
@Service
@Transactional(readOnly = true)
public class UserServiceImpl implements UserService {

    private UserRepository userRepository;
    private AccountRepository accountRepository;
    private SysParamsRepository paramsRepository;
    private PasswordEncoder passwordEncoder;
    private AccountMutexFactory accountMutexFactory;

    private final String ENCODED_PASSWORD_PREFIX = "{bcrypt}$2a$10$";

    @Lazy
    @Autowired
    private EmailService emailService;

    public UserServiceImpl(UserRepository userRepository,
                           AccountRepository accountRepository,
                           SysParamsRepository paramsRepository,
                           PasswordEncoder passwordEncoder,
                           AccountMutexFactory accountMutexFactory,
                           EmailService emailService) {
        this.userRepository = userRepository;
        this.accountRepository = accountRepository;
        this.paramsRepository = paramsRepository;
        this.passwordEncoder = passwordEncoder;
        this.accountMutexFactory = accountMutexFactory;
        this.emailService = emailService;
    }

    @Override
    public List<User> getAll(long userId) {
        log.info("Getting all for user id = {}", userId);
        return userRepository.getAll(userId);
    }

    @Override
    @Transactional
    public User create(User user, long userId) {
        log.info("Creating by user id = {} : {}", userId, user);

        checkNew(user, true);
        if (userRepository.isEmailExists(user.getEmail()))
            throw new ServerException(Exceptions.EMAIL_ALREADY_EXISTS);
        if (userRepository.isPhoneExists(user.getPhone()))
            throw new ServerException(Exceptions.PHONE_ALREADY_EXISTS);
        checkLimits(userId);
        fillNewUser(user);
        setRoles(user, userId);

        String password = user.getPassword();
        encodePassword(user);
        user = userRepository.save(user, userId);
        createAccountIfNeed(user);

        emailService.sendConfirmation(user.getEmail(), password, user.getConfirmationCode());
        return user;
    }

    @Override
    @Transactional
    public void update(User user, long userId) {
        log.info("Updating by user id = {} : {}", userId, user);
        checkNew(user, false);
        User existed = userRepository.getByEmail(user.getEmail());
        if (existed != null && !existed.getId().equals(user.getId()))
            throw new ServerException(Exceptions.EMAIL_ALREADY_EXISTS);

        if (user.getPassword() != null && !user.getPassword().startsWith(ENCODED_PASSWORD_PREFIX))
            encodePassword(user);
        checkNotFoundWithId(userRepository.save(user, userId), user.getId());
    }

    @Override
    @Transactional
    public void updateCompanyInfoId(User user, long companyInfoId) {
        log.info("Updating company info for user id = {} : {}", user.getId(), companyInfoId);
        checkNew(user, false);
        user.setCompanyInfoId(companyInfoId);
        checkNotFoundWithId(userRepository.save(user, user.getId()), user.getId());
    }

    @Override
    @Transactional
    public void delete(long id, long userId) {
        log.info("Deleting by user id = {} : user with id = {}", userId, id);
        checkNotFoundWithId(userRepository.delete(id, userId), id);
    }

    @Override
    public User get(long id, long userId) {
        log.info("Getting by user id = {} : user with id = {}", userId, id);
        return checkNotFoundWithId(userRepository.get(id, userId), id);
    }

    @Override
    @Transactional
    public void setNewPassword(String resetCode, String password) {
        log.info("Setting new password by reset code = {}", resetCode);
        User user = checkNotFound(userRepository.getByResetPasswordCode(resetCode), "resetPasswordCode=" + resetCode);
        user.setResetPasswordCode(null);
        user.setPassword(passwordEncoder.encode(password));

        userRepository.save(user, user.getId());
    }

    @Override
    @Transactional
    public void confirm(String confirm, Long userId) {
        log.info("Confirmation user with id = {}", userId);

        User user = checkNotFoundWithId(userRepository.get(userId, userId), userId);
        validateConfirmation(confirm, user);
        user.setConfirmationCode(null);
        user.setConfirmed(true);
        userRepository.save(user, userId);
    }

    @Override
    @Transactional
    public void resetPassword(String email) {
        log.info("Reset password for user with email = {}", email);

        User user = checkNotFound(userRepository.getByEmail(email.toLowerCase()), "email=" + email);
        String code = getRandomCode(UserUtil.CONFIRMATION_CODE_LENGTH);
        user.setResetPasswordCode(code);
        emailService.sendResetPassword(user.getEmail(), code);
        userRepository.save(user, user.getId());
    }

    @Override
    public Account getAccount(long userId) {
        if (userRepository.hasRole(userId, Role.ROLE_ADMIN))
            return accountRepository.get(userId);
        else if (userRepository.hasRole(userId, Role.ROLE_USER))
            return accountRepository.get(checkNotFoundWithId(userRepository.get(userId, userId).getParentId(), userId));
        return null;
    }

    @Override
    public void decreaseTestBalance(long userId) {
        Account account = accountRepository.get(userId);
        synchronized (accountMutexFactory.get(userId)) {
            account.setTestBalance(account.getTestBalance() - 1);
            if (account.getTestBalance() < 0)
                throw new ServerException(Exceptions.CAN_NOT_CREATE_MORE_TESTS);
            accountRepository.save(account);
        }
    }

    @Override
    public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException {
        log.info("Auth request for user {}", email);
        User user = userRepository.getByEmail(email.toLowerCase());
        if (user == null) {
            throw new UsernameNotFoundException("User " + email + " is not found");
        }
        return new AuthorizedUser(user);
    }

    private void validateConfirmation(String confirm, User user) {
        if (user.isConfirmed()) {
            log.info("Confirmation user with id = {} failed. Reason: {}", user.getId(), Exceptions.ALREADY_CONFIRMED.getMessage());
            throw new ServerException(Exceptions.ALREADY_CONFIRMED);
        } else if (!confirm.equals(user.getConfirmationCode())) {
            log.info("Confirmation user with id = {} failed. Reason: {}", user.getId(), Exceptions.WRONG_CONFIRMATION_CODE.getMessage());
            throw new ServerException(Exceptions.WRONG_CONFIRMATION_CODE);
        }
    }

    private void setRoles(User user, long userId) {
        Set<Role> roles = new HashSet<>();
        if (userId != 0 && userRepository.hasRole(userId, Role.ROLE_SUPERUSER)) {
            roles.add(Role.ROLE_EDITOR);
            user.setParentId(userId);
            user.setRoles(roles);
            return;
        }

        roles.add(Role.ROLE_USER);
        if (userId != 0 && userRepository.hasRole(userId, Role.ROLE_ADMIN))
            user.setParentId(userId);
        else
            roles.add(Role.ROLE_ADMIN);
        user.setRoles(roles);
    }

    private void createAccountIfNeed(User user) {
        if (user.getRoles().contains(Role.ROLE_ADMIN)) {
            SysParams params = paramsRepository.get();
            Account account = new Account();
            account.setUserId(user.getId());
            account.setTestBalance(params.getDefaultTestBalance());
            account.setSubscriptionStart(LocalDateTime.now());
            account.setSubscriptionEnd(LocalDateTime.now().plusDays(params.getDefaultSubscriptionDays()));
            account.setUserLimit(params.getDefaultUserLimit());
            accountRepository.save(account);
        }
    }

    private void checkLimits(long userId) {
        if (userId > 0 && userRepository.hasRole(userId, Role.ROLE_ADMIN)) {
            Account account = accountRepository.get(userId);
            int users = userRepository.countAll(userId);
            if (account.getUserLimit() - users <= 0)
                throw new ServerException(Exceptions.CAN_NOT_REGISTER_MORE_USERS);
        }
    }


    private void encodePassword(User user) {
        if (StringUtils.isBlank(user.getPassword()))
            return;
        user.setPassword(passwordEncoder.encode(user.getPassword()));
    }
}
