package com.example.new_test_lab.tinkoff;

import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.tinkoff.model.Payment;
import com.example.new_test_lab.tinkoff.request.BaseRequest;
import com.example.new_test_lab.tinkoff.request.InitRequest;
import com.example.new_test_lab.tinkoff.response.InitResponse;
import com.example.new_test_lab.tool.MessageHolder;
import com.example.new_test_lab.util.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;

import static com.example.new_test_lab.exception.Exceptions.CRYPTO_ERROR;


@Slf4j
public class Tinkoff {

    private static final int CONNECTION_TIMEOUT_DEFAULT = 60;
    private static final int CONNECTION_TIMEOUT_READ_DEFAULT = 60;
    @Autowired
    private MessageHolder messageHolder;

    private TinkoffParams params;
    private TinkoffApi tinkoffApi;

    public Tinkoff(TinkoffParams params) {
        this.params = params;
        checkParams();
        provideApi();
    }

    private void checkParams() {
        if (!StringUtils.hasText(params.getTinkoffUrl()) ||
                !StringUtils.hasText(params.getTinkoffPassword()) ||
                !StringUtils.hasText(params.getTinkoffTerminalKey()))
            throw new ServerException(Exceptions.PARAMS_NOT_FOUND);
    }

    public Payment initPayment(long orderId, int amount) {
        log.info("Request init payment for order {}", orderId);

        InitRequest initRequest = new InitRequest(params.getTinkoffTerminalKey(), String.valueOf(orderId), amount);
        String token = getToken(initRequest);
        initRequest.setToken(token);

        try {
            Response<InitResponse> response = tinkoffApi.init(initRequest).execute();
            return processResponse(r -> {
                log.info("Got response for init payment for order {} with code: {}", orderId, response.code());

                InitResponse initResponse = response.body();
                isSuccess(initResponse);
                return new Payment(initResponse.getPaymentURL(), String.valueOf(initResponse.getPaymentId()));
            }, response);
        } catch (IOException e) {
            throw new ServerException(Exceptions.PAYMENT_ERROR);
        }
    }

    private String getToken(BaseRequest request) {
        try {
            return TinkoffSignUtil.getSign(request);
        } catch (NoSuchAlgorithmException e) {
            throw new ServerException(CRYPTO_ERROR);
        }
    }

    private void isSuccess(InitResponse response) {
        if (!response.isSuccess())
            throw new ServerException(Exceptions.PAYMENT_ERROR.getCode(), response.getMessage());
    }

    private <T> T processResponse(Function<Response, T> f, Response response) {
        if (response.isSuccessful()) {
            return f.apply(response);
        } else {
            throw new ServerException(Exceptions.PAYMENT_ERROR, messageHolder.getMessage("test.system.error.payment.code", String.valueOf(response.code())));
        }
    }

    private void provideApi() {
        OkHttpClient.Builder builder = new OkHttpClient.Builder();
        builder.addInterceptor(HttpUtil.getLoggingInterceptor(log));
        builder.sslSocketFactory(HttpUtil.getSSLSocketFactory(), HttpUtil.geX509TrustManager());
        builder.hostnameVerifier(HttpUtil.getHostnameVerifier());

        builder.connectTimeout(CONNECTION_TIMEOUT_DEFAULT, TimeUnit.SECONDS);
        builder.readTimeout(CONNECTION_TIMEOUT_READ_DEFAULT, TimeUnit.SECONDS);
        builder.writeTimeout(CONNECTION_TIMEOUT_DEFAULT, TimeUnit.SECONDS);

        OkHttpClient client = builder.build();
        Retrofit retrofit = new Retrofit.Builder().baseUrl(params.getTinkoffUrl() + (!params.getTinkoffUrl().endsWith("/") ? "/" : "")).addConverterFactory(GsonConverterFactory.create()).client(client).build();
        tinkoffApi = retrofit.create(TinkoffApi.class);
    }
}
