package com.example.new_test_lab.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import com.google.common.base.CaseFormat;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import com.example.new_test_lab.dto.QuestionDto;
import com.example.new_test_lab.dto.RateDto;
import com.example.new_test_lab.dto.SectionTopicDto;
import com.example.new_test_lab.model.Answer;
import com.example.new_test_lab.model.CandidateAnswer;
import com.example.new_test_lab.model.CandidateQuestion;
import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.model.Rate;
import com.example.new_test_lab.model.Section;
import com.example.new_test_lab.model.Topic;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.model.enums.Position;
import com.example.new_test_lab.model.enums.QuestionType;

public class AppUtil {

    public static CandidateQuestion createCandidateQuestion(Question question, Long candidateTestId, String topicName) {
        CandidateQuestion cq = new CandidateQuestion();
        cq.setDescription(question.getDescription());
        cq.setCandidateTestId(candidateTestId);
        cq.setTopicName(topicName);
        cq.setType(question.getType());
        cq.setAnswered(false);
        return cq;
    }

    public static List<CandidateAnswer> createCandidateAnswers(List<Answer> answers, Long candidateQuestionId) {
        return answers.stream().map(a -> {
            CandidateAnswer answer = new CandidateAnswer();
            answer.setCorrect(a.isCorrect());
            answer.setCandidateQuestionId(candidateQuestionId);
            answer.setDescription(a.getDescription());
            return answer;
        }).collect(Collectors.toList());
    }

    public static String getDirection(String sort) {
        String[] sortBy = sort.split(",");
        return sortBy.length > 1 && Objects.nonNull(sortBy[1]) && sortBy[1].toLowerCase().equals("asc") ? "asc" : "desc";
    }

    public static String getSortColumnName(String sort, List<String> allowedColumn, String defaultSortBy) {
        String sortBy = sort.split(",")[0];
        String sortByUnderscore = CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sortBy);
        return allowedColumn.contains(sortByUnderscore) ? sortByUnderscore : defaultSortBy;
    }

    public static boolean isAnsweredCorrect(List<CandidateAnswer> answers, QuestionType type) {
        switch (type) {
            case SINGLE:
                return answers.stream().anyMatch(a -> a.isCorrect() && a.isSelected());
            case MULTIPLE:
                return answers.stream().noneMatch(a -> a.isCorrect() && !a.isSelected() || a.isSelected() && !a.isCorrect());
        }
        return false;
    }

    public static List<SectionTopicDto> toDto(List<Section> sections, List<Topic> topics) {
        Map<Long, List<Topic>> topicMap = topics.stream().collect(Collectors.groupingBy(Topic::getSectionId));
        return sections.stream().map(s -> new SectionTopicDto(s, topicMap.get(s.getId()))).collect(Collectors.toList());
    }

    public static QuestionDto toDto(Question question, List<Answer> answers) {
        return new QuestionDto(question, answers);
    }

    public static int countPages(int count, int size) {
        return count / size + ( count % size > 0 ? 1 : 0 );
    }

    public static String getRandomCode(int length) {
        return RandomStringUtils.random(length, true, true);
    }

    public static List<RateDto> toDto(List<Rate> rateList) {
        if (rateList.isEmpty()) return Collections.emptyList();
        double maxTestPrice = rateList.stream().mapToDouble(r -> r.getBillPrice() / r.getTestAmount()).max().getAsDouble();
        return rateList.stream().map(r -> {
            double testPrice = r.getBillPrice() / r.getTestAmount();
            double profit = r.getTestAmount() * maxTestPrice - r.getBillPrice();
            return new RateDto(r, testPrice, profit);
        }).collect(Collectors.toList());
    }

    public static List<String> getLevelsFilter(String level) {
        if (level == null || StringUtils.isBlank(level)) return null;
        List<String> levelCollection = new ArrayList<>(
                Arrays.asList(level.trim().toUpperCase().split(", *"))
        );
        levelCollection.retainAll(Level.stringValuesList());
        return levelCollection;
    }

    public static List<String> getPositionsFilter(String position) {
        if (position == null || StringUtils.isBlank(position)) return null;
        List<String> positionCollection = new ArrayList<>(
                Arrays.asList(position.trim().toUpperCase().split(", *"))
        );
        positionCollection.retainAll(Position.stringValuesList());
        return positionCollection;
    }

    public static List<String> getStatusesFilter(String status) {
        if (status == null || StringUtils.isBlank(status)) return null;
        List<String> statusCollection = new ArrayList<>(
                Arrays.asList(status.trim().toUpperCase().split(", *"))
        );
        statusCollection.retainAll(CandidateTestStatus.stringValuesList());
        return statusCollection;
    }

}
