package com.example.new_test_lab.util;

import com.example.new_test_lab.dto.CreateCandidateTestDto;
import com.example.new_test_lab.exception.Exceptions;
import com.example.new_test_lab.exception.ServerException;
import com.example.new_test_lab.model.CandidateTest;
import com.example.new_test_lab.model.Question;
import com.example.new_test_lab.model.Test;
import com.example.new_test_lab.model.enums.CandidateTestStatus;
import com.example.new_test_lab.model.enums.Level;
import com.example.new_test_lab.repository.QuestionRepository;
import com.google.common.primitives.Longs;


import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static com.example.new_test_lab.util.AppUtil.getRandomCode;


public class TestUtil {

    public final static int TEST_LINK_CODE_LENGTH = 32;

    public static CandidateTest fromDto(CreateCandidateTestDto dto) {
        CandidateTest candidateTest = new CandidateTest();
        candidateTest.setTestId(dto.getTestId());
        candidateTest.setCandidateName(dto.getCandidateName());
        candidateTest.setNote(dto.getNote());
        return candidateTest;
    }

    public static void fillIn(Test test, Test testDb) {
        testDb.setDescription(test.getDescription());
        testDb.setName(test.getName());
        testDb.setDuration(test.getDuration());
        testDb.setIncreasedComplexity(test.isIncreasedComplexity());
    }

    public static long[] pickUpQuestionsByLevel(List<Long> topicIds, int questionNumber, Level level, QuestionRepository repository) {
        if (questionNumber > repository.countAllByLevelAndTopicIds(level, Longs.toArray(topicIds)))
            throw new ServerException(Exceptions.NOT_ENOUGH_QUESTIONS_FOR_TEST);

        List<Question> questionList = repository.getAllByLevelAndTopicIds(level, topicIds.stream().mapToLong(Long::longValue).toArray());
        Collections.shuffle(questionList);
        List<List<Question>> questions = questionList.stream().collect(Collectors.groupingBy(Question::getTopicId))
                .values().stream().sorted(Comparator.comparingInt(List::size)).collect(Collectors.toList());

        AtomicInteger topicCount = new AtomicInteger(topicIds.size());
        final int questionsPerTopic = questionNumber / topicCount.get();
        AtomicInteger lack = new AtomicInteger(questionNumber % topicCount.get());

        return questions.stream().map(l -> {
            List<Question> list;
            if (l.size() < questionsPerTopic) {
                lack.addAndGet(questionsPerTopic - l.size());
                list = l;
            } else {
                int delta = lack.get() / topicCount.get();
                if (l.size() < questionsPerTopic + delta)
                    delta = questionsPerTopic + delta - l.size();
                list = l.subList(0, questionsPerTopic + delta);
                lack.addAndGet(-delta);
            }
            topicCount.decrementAndGet();
            return list;
        }).flatMap(List::stream).mapToLong(Question::getId).toArray();
    }

    public static long[] pickUpQuestionsByComplexity(List<Long> topicIds, int questionNumber, boolean increasedComplexity, QuestionRepository repository) {
        if (increasedComplexity && questionNumber > repository.countAllByTopicIds(Longs.toArray(topicIds)) ||
            !increasedComplexity && questionNumber > repository.countAllByComplexityAndTopicIds(increasedComplexity, Longs.toArray(topicIds)))
            throw new ServerException(Exceptions.NOT_ENOUGH_QUESTIONS_FOR_TEST);

        List<Question> questionList = new ArrayList<>();
        List<Question> complexQuestionList = new ArrayList<>();
        List<Question> defaultQuestionList = repository.getAllByComplexityAndTopicIds(false, topicIds.stream().mapToLong(Long::longValue).toArray());
        Collections.shuffle(defaultQuestionList);
        if (increasedComplexity) {
            complexQuestionList = repository.getAllByComplexityAndTopicIds(increasedComplexity,
                                                     topicIds.stream().mapToLong(Long::longValue).toArray());
            Collections.shuffle(complexQuestionList);
            complexQuestionList = complexQuestionList.stream().limit(questionNumber/2).collect(Collectors.toList());
            defaultQuestionList = defaultQuestionList.stream().limit(questionNumber - complexQuestionList.size()).collect(Collectors.toList());
        } else {
            defaultQuestionList = defaultQuestionList.stream().limit(questionNumber).collect(Collectors.toList());
        }
        questionList.addAll(defaultQuestionList);
        questionList.addAll(complexQuestionList);
        Collections.shuffle(questionList);
        List<List<Question>> questions = questionList.stream().collect(Collectors.groupingBy(Question::getTopicId))
                .values().stream().sorted(Comparator.comparingInt(List::size)).collect(Collectors.toList());

        AtomicInteger topicCount = new AtomicInteger(topicIds.size());
        final int questionsPerTopic = questionNumber / topicCount.get();
        AtomicInteger lack = new AtomicInteger(questionNumber % topicCount.get());

        return questions.stream().map(l -> {
            List<Question> list;
            if (l.size() < questionsPerTopic) {
                lack.addAndGet(questionsPerTopic - l.size());
                list = l;
            } else {
                int delta = lack.get() / topicCount.get();
                if (l.size() < questionsPerTopic + delta)
                    delta = questionsPerTopic + delta - l.size();
                list = l.subList(0, questionsPerTopic + delta);
                lack.addAndGet(-delta);
            }
            topicCount.decrementAndGet();
            return list;
        }).flatMap(List::stream).mapToLong(Question::getId).toArray();
    }

    public static CandidateTest fillNewCandidateTest(Test test, CandidateTest candidateTest) {
        candidateTest.setTestId(test.getId());
        candidateTest.setDuration(test.getDuration());
        candidateTest.setName(test.getName());
        candidateTest.setDescription(test.getDescription());
        candidateTest.setLinkCode(getRandomCode(TEST_LINK_CODE_LENGTH));
        candidateTest.setStatus(CandidateTestStatus.NEW);
        candidateTest.setUserId(test.getUserId());
        candidateTest.setQuestionNumber(test.getQuestionNumber());
        candidateTest.setCreated(LocalDateTime.now());
        candidateTest.setLevel(test.getLevel());
        candidateTest.setPosition(test.getPosition());
        candidateTest.setIncreasedComplexity(test.isIncreasedComplexity());
        return candidateTest;
    }
}
