import React from "react";
import {connect} from "react-redux";
import {Icon, IconButton, Schema, Table, Tooltip, Whisper} from "rsuite";
import {testActions} from "../../actions/test.actions";
import {language} from "../../content/language";
import {alertActions} from "../../actions/alert.actions";
import {history} from "../../helpers/history";
import {styleConstants} from "../../constants/style.constants";
import linkIcon from "../../images/link_icon.svg"
import candidateTestResultButton from "../../images/candidate_test_details_button.svg"
import {renderLevel} from "../../helpers/level.render";
import loyaltyLog from "../../helpers/loyaltyLog";


const { Column, HeaderCell, Cell, Pagination } = Table;


class CandidateTestsList extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT CANDIDATE TEST LIST");
        const {displayLength, page, sortColumn, sortType, formValue} = this.state;

        this.props.dispatch(testActions.getCandidateTests(page, displayLength, sortColumn, sortType));
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT CANDIDATE TEST LIST");
        super(props);

        this.state = {
            displayLength: 10,
            loading: false,
            page: 1,
            sortColumn: 'created',
            sortType: 'desc',
            formValue: {
                id: '',
                statuses: []
            },
            testsStatuses: [],
            showFilter: false
        };
    }

    handleChangePage = (page) => {
        const { sortColumn, sortType, displayLength, formValue } = this.state;

        this.setState({
            page: page
        });
        this.props.dispatch(testActions.getCandidateTests(page, displayLength, sortColumn, sortType));
    };

    handleChangeLength = (displayLength) => {
        const { page, sortColumn, sortType, formValue } = this.state;

        this.setState({
            page: page,
            displayLength: displayLength
        });
        this.props.dispatch(testActions.getCandidateTests(page, displayLength, sortColumn, sortType));
    };

    handleSortColumn = (sortColumn, sortType) => {
        const { page, displayLength } = this.state;

        this.setState({
            sortColumn,
            sortType
        });
        //get last elem splitted by dot cause column name is candidateTest.name etc
        this.props.dispatch(testActions.getCandidateTests(page, displayLength, sortColumn.split('.').pop(), sortType));
    };

    toggleFilter = () => {
        this.setState({
            showFilter: !this.state.showFilter
        })
    };

    filterAction = () => {
        const { displayLength, page, sortColumn, sortType, formValue } = this.state;
        if (!this.testFilterForm.check()) {
            return;
        }
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    clearFilterInputs = () => {
        const { displayLength, page, sortColumn, sortType } = this.state;
        let filterFormValue = {
            id: '',

        };
        this.setState({ formValue: filterFormValue });
        this.props.dispatch(testActions.getCandidateTests(page, displayLength, sortColumn, sortType));
    };

    render() {

        loyaltyLog("RENDER COMPONENT CANDIDATE TEST LIST");

        const { candidateTestsList } = this.props;
        const { displayLength, page, sortColumn, sortType, formValue, formError } = this.state;
        const hasEditPermit = true;

        return (
            <div>
                <h3 className={"ll-table-header"}>{language.CANDIDATE_TESTS_LIST}</h3>

                <Table autoHeight data={candidateTestsList.items} loading={candidateTestsList.loading}
                       headerHeight={styleConstants.TABLE_HEADER_ROW_HEIGHT}
                       rowHeight={styleConstants.TABLE_BODY_ROW_HEIGHT}
                       sortColumn={sortColumn}
                       sortType={sortType}
                       onSortColumn={this.handleSortColumn}
                       renderEmpty={() => <div className="rs-table-body-info">{language.NO_CANDIDATE_TESTS}</div>}>>


                    <Column width={170} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.CANDIDATE_FIO}</HeaderCell>
                        <Cell dataKey="candidateName"/>
                    </Column>

                    <Column width={200} resizable sortable>
                        <HeaderCell>{language.TEST_TITLE}</HeaderCell>
                        <Cell dataKey="name"/>
                    </Column>

                    <Column width={180} align="center" sortable>
                        <HeaderCell>{language.LEVEL}</HeaderCell>
                        <Cell dataKey="level">
                            {rowData => renderLevel(rowData['level']) }
                        </Cell>
                    </Column>

                    <Column align="center" width={190} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.TEST_STARTED_DATE}</HeaderCell>
                        <CandidateTestDateCell dataKey="testStarted" />
                    </Column>

                    <Column align="center" width={190} resizable sortable style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.TEST_FINISHED_DATE}</HeaderCell>
                        <CandidateTestDateCell dataKey="testFinished" />
                    </Column>

                    <Column align="center" width={130} fixed="right">
                        <HeaderCell>{language.TEST_LINK_CODE}</HeaderCell>
                        <CandidateTestLinkCodeCell dispatch={this.props.dispatch}/>
                    </Column>

                    <Column align="center" width={100} fixed="right" style={{fontWeight: 'bold'}}>
                        <HeaderCell>{language.TEST_RESULT}</HeaderCell>
                        <CandidateTestAnswersStatsCell questionNum={'questionNumber'} correctAnswers={'correctQuestionNumber'}/>
                    </Column>

                    <Column width={121} fixed="right">
                        <HeaderCell/>
                        <CandidateTestDetailsCell dataKey={'id'}/>
                    </Column>

                </Table>
                <Pagination
                    lengthMenu={[
                        {
                            value: 10,
                            label: 10
                        },
                        {
                            value: 20,
                            label: 20
                        }
                    ]}
                    activePage={page}
                    displayLength={displayLength}
                    total={candidateTestsList.totalElements}
                    onChangePage={this.handleChangePage}
                    onChangeLength={this.handleChangeLength}
                />
            </div>
        );
    }
}

const { NumberType } = Schema.Types;
const testFilterModel = Schema.Model({

});

const CandidateTestAnswersStatsCell = ({rowData, questionNum, correctAnswers, dataKey, ...props}) => {
    return (
        <Cell {...props} >
            {rowData['status'] === 'FINISHED' ? rowData[correctAnswers] + ' / ' + rowData[questionNum] : '-'}
        </Cell>
    );
};

const CandidateTestDateCell = ({rowData, dataKey, ...props}) => {
    return (
        <Cell {...props} className="link-group">
            {rowData[dataKey] ? new Date(rowData[dataKey]).toLocaleString() : ''}
        </Cell>
    );
};

const CandidateTestLinkCodeCell = ({ rowData, dispatch, ...props }) => {

    function handleAction() {
        //TODO copy(location.origin + "/test-session/" + link);
        var dummy = document.createElement("textarea");
        // to avoid breaking orgain page when copying more words
        // cant copy when adding below this code
        // dummy.style.display = 'none'
        document.body.appendChild(dummy);
        //Be careful if you use texarea. setAttribute('value', value), which works with "input" does not work with "textarea".
        //TODO fix it in the future
        dummy.value = 'https://testlab.loylabs.ru:4201/test-session/' + rowData['linkCode'];
        dummy.select();
        document.execCommand("copy");
        dispatch(alertActions.success(language.TEST_LINK_CODE_COPIED, 20000));
        document.body.removeChild(dummy);
    }
    return (
        <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
            <Whisper placement="top" trigger="hover" speaker={<Tooltip>{language.COPY_LINK_CODE_TOOLTIP}</Tooltip>}>
            <IconButton
                appearance="subtle"
                onClick={handleAction}
                icon={<Icon icon={linkIcon}/>}
            />
            </Whisper>
        </Cell>
    );
};

const CandidateTestDetailsCell = ({ rowData, dataKey, ...props }) => {
    function handleAction() {
        history.push(`/candidate-tests/${rowData[dataKey]}`);
    }
    return (
        <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
            <IconButton className={"ll-candidate-test-result-button"}
                appearance="subtle"
                onClick={handleAction}
                icon={<Icon icon={candidateTestResultButton}/>}
            />
        </Cell>
    );
};


function mapStateToProps(state) {

    const { candidateTestsList} = state;
    return {
        candidateTestsList
    };
}

const connectedCandidateTestsList = connect(mapStateToProps)(CandidateTestsList);
export { connectedCandidateTestsList as CandidateTestsList };