import React from "react";
import {connect} from "react-redux";
import {
    Button,
    Col,
    Icon,
    IconButton, Modal, Row, Schema,
    Table
} from "rsuite";
import {questionActions} from "../../actions/question.actions";
import { history} from "../../helpers/history";
import {language} from "../../content/language";
import {CreateObjectButton} from "../buttons/CreateObjectButton";
import {DeleteCell} from "../buttons/DeleteCell";
import {responseMessages} from "../../content/responseMessages";
import {styleConstants} from "../../constants/style.constants";
import createQuestionButton from "../../images/create_question_button.svg";
import {pathConstants} from "../../constants/path.constants";
import {BackTextLink} from "../buttons/BackTextLink";
import editIcon from "../../images/edit_icon.svg";
import star1 from "../../images/star1.svg";
import star2 from "../../images/star2.svg";
import star3 from "../../images/star3.svg";
import {renderLevel} from "../../helpers/level.render";
import loyaltyLog from "../../helpers/loyaltyLog";

const { Column, HeaderCell, Cell, Pagination } = Table;


class QuestionsList extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT QUESTIONS LIST");
        const {displayLength, page, sortColumn, sortType, formValue} = this.state;

        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    }

    componentDidUpdate(prevProps, prevState, snapshot) {

        const { sortColumn, sortType, displayLength } = this.state;
        const statePage = this.state.page;

        //если все элементы на странице удалены и страница по номеру больше 1, перейти на прерыдущую страницу
        if (statePage > 1
            && this.props.questionsList.items.length === 0
            && prevProps.questionsList.items.length - this.props.questionsList.items.length === 1) {

            this.setState({
                page: statePage - 1
            });
            //go to prev page
            this.props.dispatch(
                questionActions.getQuestions(this.props.match.params.id, statePage - 1, displayLength, sortColumn, sortType)
            );
        }
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT QUESTIONS LIST");
        super(props);

        this.state = {
            displayLength: 10,
            loading: false,
            page: 1,
            sortColumn: 'id',
            sortType: 'asc',
            formValue: {
                id: '',
                statuses: []
            },
            qStatuses: [],
            showFilter: false
        };
    }

    handleChangePage = (page) => {
        const { sortColumn, sortType, displayLength, formValue } = this.state;

        this.setState({
            page: page
        });
        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    };

    handleChangeLength = (displayLength) => {
        const { page, sortColumn, sortType, formValue } = this.state;

        this.setState({
            page: page,
            displayLength: displayLength
        });
        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    };

    handleSortColumn = (sortColumn, sortType) => {
        const { page, displayLength,formValue } = this.state;

        this.setState({
            sortColumn,
            sortType
        });
        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    };

    toggleFilter = () => {
        this.setState({
            showFilter: !this.state.showFilter
        })
    };

    filterAction = () => {
        const { displayLength, page, sortColumn, sortType, formValue } = this.state;
        if (!this.questionFilterForm.check()) {
            return;
        }
        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    };

    clearFilterInputs = () => {
        const { displayLength, page, sortColumn, sortType } = this.state;
        let filterFormValue = {
            id: '',
            name: '',
            address: '',
            serialNumber: '',
            ratingFrom: '',
            ratingTo: '',
            statuses: []
        };
        this.setState({ formValue: filterFormValue });
        this.props.dispatch(questionActions.getQuestions(this.props.match.params.id, page, displayLength, sortColumn, sortType));
    };

    render() {

        loyaltyLog("RENDER COMPONENT QUESTIONS LIST");

        const { questionsList } = this.props;
        const { displayLength, page, sortColumn, sortType, formValue, questionsStatuses, formError } = this.state;

        const hasEditPermit = true;

        return (
            <div>

                <Row className={"ll-table-header"}>
                    <Col xs={12}>
                        <h3>{language.QUESTIONS_LIST}</h3>
                    </Col>
                    <Col xs={12}>
                        <CreateObjectButton hasEditPermit={hasEditPermit} icon={<Icon icon={createQuestionButton}/>}
                                            className={"create-question-button"}
                                            onClickAction={() => history.push(`/sections/topics/${this.props.match.params.id}/questions/create`)}/>
                    </Col>

                </Row>
                <BackTextLink action={() => history.push(pathConstants.TOPICS_LIST)} text={language.BACK_TO_TOPICS} />

                <Table autoHeight data={questionsList.items} loading={questionsList.loading}
                       headerHeight={styleConstants.TABLE_HEADER_ROW_HEIGHT}
                       rowHeight={styleConstants.TABLE_BODY_ROW_HEIGHT}
                       sortColumn={sortColumn}
                       sortType={sortType}
                       onSortColumn={this.handleSortColumn}
                       style={{marginTop: 20}}>

                    <Column width={55} resizable sortable style={{paddingLeft: 0}}>
                        <HeaderCell>{language.ID}</HeaderCell>
                        <Cell dataKey="id" />
                    </Column>

                    <Column minWidth={350} flexGrow={2} sortable>
                        <HeaderCell>{language.DESCRIPTION}</HeaderCell>
                        <Cell>
                            {rowData => (<div dangerouslySetInnerHTML={{__html: rowData.description}}/> )}
                        </Cell>
                    </Column>

                    <Column width={180} align="center" sortable>
                        <HeaderCell>{language.QUESTION_LEVEL}</HeaderCell>
                        <Cell dataKey="level">
                            {rowData => renderLevel(rowData['increasedComplexity']) }
                        </Cell>
                    </Column>

                    <Column width={80} fixed="right">
                        <HeaderCell/>
                        <QuestionDetailsCell dataKey={'id'} topicId={this.props.match.params.id}/>
                    </Column>

                    <Column width={90} fixed="right">
                        <HeaderCell/>
                        <QuestionDeleteCell dataKey={'id'} dispatch={this.props.dispatch} hasEditPermit={hasEditPermit}/>
                    </Column>

                </Table>
                <Pagination
                    lengthMenu={[
                        {
                            value: 10,
                            label: 10
                        },
                        {
                            value: 20,
                            label: 20
                        }
                    ]}
                    activePage={page}
                    displayLength={displayLength}
                    total={questionsList.totalElements}
                    onChangePage={this.handleChangePage}
                    onChangeLength={this.handleChangeLength}
                />
            </div>
        );
    }
}

const { NumberType } = Schema.Types;
const questionFilterModel = Schema.Model({
    id: NumberType(language.INSERT_NUMBER)
});

const QuestionDetailsCell = ({ topicId, rowData, dataKey, ...props }) => {
    function handleAction() {
        history.push(`questions/${rowData[dataKey]}`);
    }
    return (
        <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
            <IconButton
                appearance="subtle"
                onClick={handleAction}
                icon={<Icon icon={editIcon} />}
            />
        </Cell>
    );
};

const QuestionDeleteCell = ({ rowData, dataKey, dispatch, hasEditPermit, ...props }) => {
    const handleAction = () => {
        // eslint-disable-next-line no-restricted-globals
        dispatch(
            questionActions.deleteQuestion(`${rowData[dataKey]}`)
        );
        setModalShow(false);
    };
    const [modalShow, setModalShow] = React.useState(false);
    return (
        <div>
            <DeleteCell {...props} onClickAction={() => setModalShow(true)} hasEditPermit={hasEditPermit}/>
            <Modal show={modalShow} onHide={() => setModalShow(false)}>
                <Modal.Body>
                    <p>{responseMessages.QUESTION_DELETE_CONFIRM}</p>
                </Modal.Body>
                <Modal.Footer>
                    <Button onClick={handleAction} appearance="primary">{language.OK}</Button>
                    <Button onClick={() => setModalShow(false)} appearance="subtle">{language.CANCEL}</Button>
                </Modal.Footer>
            </Modal>
        </div>
    );
};

function mapStateToProps(state) {

    const { questionsList} = state;
    return {
        questionsList
    };
}

const connectedQuestionsList = connect(mapStateToProps)(QuestionsList);
export { connectedQuestionsList as QuestionsList };