import React, {Component} from "react";
import {connect} from "react-redux";
import {
    Button, Col,
    ControlLabel, Form, FormControl, FormGroup,
    Icon,
    IconButton, Modal, Row, Schema,
    Table
} from "rsuite";
import editIcon from "../../images/edit_icon.svg";
import testUrlButton from "../../images/test_url_button.svg";
import {testActions} from "../../actions/test.actions";
import { history} from "../../helpers/history";
import {language} from "../../content/language";
import {CreateObjectButton} from "../buttons/CreateObjectButton";
import {pathConstants} from "../../constants/path.constants";
import {DeleteCell} from "../buttons/DeleteCell";
import {styleConstants} from "../../constants/style.constants";
import createTestButton from "../../images/create_test_button.svg";
import {StringType} from "schema-typed";
import {renderLevel} from "../../helpers/level.render";
import loyaltyLog from "../../helpers/loyaltyLog";

const { Column, HeaderCell, Cell, Pagination } = Table;


class TestsList extends React.Component {

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT TEST LIST");
        const {displayLength, page, sortColumn, sortType} = this.state;

        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    }

    constructor(props) {
        loyaltyLog("CONSTRUCTOR COMPONENT TEST LIST");
        super(props);

        this.state = {
            displayLength: 10,
            loading: false,
            page: 1,
            sortColumn: 'id',
            sortType: 'asc',
            formValue: {
                id: '',
                statuses: []
            },
            testsStatuses: [],
            showFilter: false,
            showCandidateModal: false,
            modalCandidateTestId: ''
        };
    }

    componentDidUpdate(prevProps, prevState, snapshot) {

        const { sortColumn, sortType, displayLength } = this.state;
        const statePage = this.state.page;

        //если все элементы на странице удалены и страница по номеру больше 1, перейти на прерыдущую страницу
        if (statePage > 1
            && this.props.testsList.items.length === 0
            && prevProps.testsList.items.length - this.props.testsList.items.length === 1) {

            this.setState({
                page: statePage - 1
            });
            //go to prev page
            this.props.dispatch(
                testActions.getTests(statePage - 1, displayLength, sortColumn, sortType)
            );
        }
    }

    handleChangePage = (page) => {
        const { sortColumn, sortType, displayLength } = this.state;

        this.setState({
            page: page
        });
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    handleChangeLength = (displayLength) => {
        const { page, sortColumn, sortType } = this.state;

        this.setState({
            page: page,
            displayLength: displayLength
        });
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    handleSortColumn = (sortColumn, sortType) => {
        const { page, displayLength } = this.state;

        this.setState({
            sortColumn,
            sortType
        });
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    toggleFilter = () => {
      this.setState({
          showFilter: !this.state.showFilter
      })
    };

    filterAction = () => {
        const { displayLength, page, sortColumn, sortType } = this.state;
        if (!this.testFilterForm.check()) {
            return;
        }
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    clearFilterInputs = () => {
        const { displayLength, page, sortColumn, sortType } = this.state;
        let filterFormValue = {
            id: '',
            name: '',
            address: '',
            serialNumber: '',
            ratingFrom: '',
            ratingTo: '',
            statuses: []
        };
        this.setState({ formValue: filterFormValue });
        this.props.dispatch(testActions.getTests(page, displayLength, sortColumn, sortType));
    };

    render() {

        loyaltyLog("RENDER COMPONENT TEST LIST");

        const { testsList } = this.props;
        const { displayLength, page, sortColumn, sortType, showCandidateModal, modalCandidateTestId } = this.state;

        const hasEditPermit = true;

        return (
            <div>
                <Row className={"ll-table-header"}>
                    <Col xs={12}>
                        <h3>{language.TESTS_LIST}</h3>
                    </Col>
                    <Col xs={12}>
                        <CreateObjectButton hasEditPermit={hasEditPermit} icon={<Icon icon={createTestButton}/>}
                                            className={"create-test-button"}
                                            onClickAction={() => history.push(pathConstants.TEST_CREATE)}/>
                    </Col>
                </Row>

                <Table autoHeight data={testsList.items} loading={testsList.loading}
                       headerHeight={styleConstants.TABLE_HEADER_ROW_HEIGHT}
                       rowHeight={styleConstants.TABLE_BODY_ROW_HEIGHT}
                       sortColumn={sortColumn} sortType={sortType}
                       onSortColumn={this.handleSortColumn}
                       renderEmpty={() => <div className="rs-table-body-info">{language.NO_TESTS}</div>}>

                    <Column minWidth={350} flexGrow={1} sortable>
                        <HeaderCell>{language.TEST_TITLE}</HeaderCell>
                        <Cell dataKey="name" style={{fontWeight: 600}}/>
                    </Column>

                    <Column width={180} align="center" sortable>
                        <HeaderCell>{language.LEVEL}</HeaderCell>
                        <Cell dataKey="level">
                            {rowData => renderLevel(rowData['increasedComplexity']) }
                        </Cell>
                    </Column>

                    <Column align="center" width={185} resizable sortable>
                        <HeaderCell>{language.DURATION_MINUTES}</HeaderCell>
                        <Cell>{rowData => Math.floor(rowData.duration / 60)}</Cell>
                    </Column>

                    <Column align="center" width={70} fixed="right" style={{background: 'white'}}>
                        <HeaderCell/>
                        <TestDetailsCell dataKey={'id'} />
                    </Column>

                    <Column width={70} fixed="right" style={{background: 'white'}}>
                        <HeaderCell/>
                        <TestDeleteCell dataKey={'id'} dispatch={this.props.dispatch} hasEditPermit={hasEditPermit}/>
                    </Column>

                    <Column width={195} fixed="right" style={{background: 'white', paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
                        <HeaderCell/>
                        <Cell dataKey="id">
                            {rowData => (
                                <IconButton className={"test-url-button"}
                                        appearance="subtle"
                                        onClick={() => this.setState({
                                            showCandidateModal: true,
                                            modalCandidateTestId: rowData.id
                                        })}
                                        icon={<Icon icon={testUrlButton}/>}
                                />
                            )}
                        </Cell>
                    </Column>

                </Table>
                <Pagination
                    lengthMenu={[
                        {
                            value: 10,
                            label: 10
                        },
                        {
                            value: 20,
                            label: 20
                        }
                    ]}
                    activePage={page}
                    displayLength={displayLength}
                    total={testsList.totalElements}
                    onChangePage={this.handleChangePage}
                    onChangeLength={this.handleChangeLength}
                />
                <TestGeneratorModal showCandidateModal={showCandidateModal} testId={modalCandidateTestId} dispatch={this.props.dispatch}/>
            </div>
        );
    }
}

const { NumberType } = Schema.Types;
const testFilterModel = Schema.Model({
    id: NumberType(language.INSERT_NUMBER)
});

class TestGeneratorModal extends Component {

    constructor(props) {
        super(props);
        this.state = {
            show: false,
            modalFormValue: {
                candidateName: ''
            },
            modalFormError: {},
            testId: ''
        }
    }

    UNSAFE_componentWillReceiveProps(props) {
        this.setState({
            show: props.showCandidateModal,
            testId: props.testId
        })
    }

    render() {

        loyaltyLog("RENDER TEST MODAL");

        const { modalFormValue, modalFormError } = this.state;

        return (<Modal show={this.state.show} onHide={() => this.setState({show: false})}>
            <Modal.Header>
                <Modal.Title>{language.CANDIDATE_TEST_GEN_TITLE}</Modal.Title>
            </Modal.Header>
            <Modal.Body>
                <p>{language.CANDIDATE_TEST_GEN_BODY}</p>
                <Form style={{padding: 15}}
                      formValue={modalFormValue}
                      formError={modalFormError}
                      onChange={modalFormValue => {
                          this.setState({modalFormValue});
                      }}
                      onCheck={modalFormError => this.setState({modalFormError})}
                      ref={ref => (this.form = ref)}
                      model={Schema.Model({candidateName: StringType().isRequired(language.REQUIRED_INPUT)})}
                      checkTrigger={'blur'}>

                    <FormGroup>
                        <ControlLabel>{language.CANDIDATE_FIO}</ControlLabel>
                        <FormControl name="candidateName" style={{width: 500}}/>
                    </FormGroup>
                </Form>
            </Modal.Body>
            <Modal.Footer>
                <Button onClick={() => {
                    if (!this.form.check()) {
                        return
                    }
                    this.props.dispatch(testActions.createCandidateTest(this.state.testId, modalFormValue.candidateName,
                        () => this.setState({show: false, modalFormValue: { candidateName: ''}})));
                }} appearance="primary">
                    {language.COPY_LINK}
                </Button>
                <Button onClick={() => this.setState({show: false, modalFormValue: { candidateName: ''}})} appearance="subtle">
                    {language.CANCEL}
                </Button>
            </Modal.Footer>
        </Modal>)
    }
}

const TestDetailsCell = ({ rowData, dataKey, ...props }) => {
    function handleAction() {
        history.push(`/tests/${rowData[dataKey]}`);
    }
    return (
        <Cell {...props} style={{paddingTop: styleConstants.TABLE_BODY_CONTENT_PADDING_VERTICAL}}>
            <IconButton
                appearance="subtle"
                onClick={handleAction}
                icon={<Icon icon={editIcon}/>}
            />
        </Cell>
    );
};

const TestDeleteCell = ({ rowData, dataKey, dispatch, hasEditPermit, ...props }) => {
    const handleAction = () => {
        // eslint-disable-next-line no-restricted-globals
        dispatch(
            testActions.deleteTest(`${rowData[dataKey]}`)
        );
        setModalShow(false);
    };
    const [modalShow, setModalShow] = React.useState(false);
    return (
        <div>
            <DeleteCell {...props} onClickAction={() => setModalShow(true)} hasEditPermit={hasEditPermit}/>
            <Modal show={modalShow} onHide={() => setModalShow(false)}>
                <Modal.Body>
                    <p>{language.ARE_YOU_SURE_TO_DELETE_TEST}</p>
                </Modal.Body>
                <Modal.Footer>
                    <Button onClick={handleAction} appearance="primary">{language.OK}</Button>
                    <Button onClick={() => setModalShow(false)} appearance="subtle">{language.CANCEL}</Button>
                </Modal.Footer>
            </Modal>
        </div>
    );
};

function mapStateToProps(state) {

    const { testsList} = state;
    return {
        testsList
    };
}

const connectedTestsList = connect(mapStateToProps)(TestsList);
export { connectedTestsList as TestsList };