import React from 'react';
import {connect} from 'react-redux';

import {SideNav} from "../components/SideNav";

import {Col, Grid, Icon, Nav, Navbar, Row} from "rsuite";
import 'rsuite/dist/styles/rsuite-default.css';
import {PrivateRouteView} from "../routes/PrivateRouteView";
import NotFoundPage from "./NotFoundPage";
import {Route, Switch} from "react-router-dom";
import {pathConstants} from "../constants/path.constants";
import {TestsList} from "../components/tests/TestsList";
import {TestDetails} from "../components/tests/TestDetails";
import {SectionsList} from "../components/topics/SectionsList";
import {QuestionsList} from "../components/questions/QuestionsList";
import {QuestionDetails} from "../components/questions/QuestionDetails";
import {language} from "../content/language";
import {CandidateTestsList} from "../components/candidate-tests/CandidateTestsList";
import {CandidateTestDetails} from "../components/candidate-tests/CandidateTestDetails";
import {store} from "../helpers/store";
import {userActions} from "../actions/user.actions";
import {history} from "../helpers/history";
import {UserSettings} from "../components/account/UserSettings";
import {hasAuthority} from "../helpers/auth";
import {authoritiesConstants} from "../constants/authorities.constants";
import PurchaseAccount from "../images/purchase_account.svg";
import {DepositScreen} from "../components/account/DepositScreen";
import loyaltyLog from "../helpers/loyaltyLog";
import {userService} from "../services/user.service";
import {TopicsList} from "../components/topics/TopicsList";
import {CompanyInfo} from "../components/account/CompanyInfo";
import {OrdersList} from "../components/orders/OrdersList";
import {orderService} from "../services/order.service";


class SideBarLayout extends React.Component {

    constructor(props) {
        loyaltyLog('CONSTRUCTOR SideBarLayout COMPONENT');
        super(props);
        this.state = {
            hasTestViewPermit: hasAuthority(authoritiesConstants.ROLE_USER),
            hasCandidateTestViewPermit: hasAuthority(authoritiesConstants.ROLE_USER),
            hasSectionViewPermit: hasAuthority(authoritiesConstants.ROLE_EDITOR),
            hasTopicsViewPermit: hasAuthority(authoritiesConstants.ROLE_EDITOR),
            hasQuestionViewPermit: hasAuthority(authoritiesConstants.ROLE_EDITOR),
            hasBalance: hasAuthority(authoritiesConstants.ROLE_ADMIN),
            balance: '',
            subscriptionUntil: '',
            isSubscriptionActive: true,
        }
    }

    componentDidMount() {
        loyaltyLog("componentDidMount COMPONENT SIDEBAR LAYOUT");

        if (this.state.hasBalance) {
            userService.getBalance().then(response => {
                const dateLong = response.data.subscriptionUntil;
                const booleanSubscriptionActive = dateLong > Date.now();
                localStorage.setItem('isSubscriptionActive', booleanSubscriptionActive);
                const date = this.toDate(dateLong);
                this.setState({   isSubscriptionActive: booleanSubscriptionActive,
                                        balance: response.data.testBalance,
                                        subscriptionUntil: date,
                                        hasTestViewPermit: hasAuthority(authoritiesConstants.ROLE_USER) && booleanSubscriptionActive ,
                                        hasCandidateTestViewPermit: hasAuthority(authoritiesConstants.ROLE_USER) && booleanSubscriptionActive});
            })
        }
    };

    handleLogout = () => {
        store.dispatch(userActions.logout());
    };

    toDate(dateLong) {
        const DATE_OPTIONS = { year: 'numeric', month: 'long', day: 'numeric' };
        return dateLong > Date.now() ? language.BEFORE + (new Date(dateLong)).toLocaleDateString('ru-RU', DATE_OPTIONS) : language.SUBSCRIPTION_EXPIRED;
    }

    render() {

        const { hasTestViewPermit, hasCandidateTestViewPermit, hasSectionViewPermit, hasQuestionViewPermit,
            hasBalance, balance, subscriptionUntil, isSubscriptionActive, hasTopicsViewPermit } = this.state;
        return (
            <Grid fluid>

                <Row>
                    <Col xs={24} sm={24} md={6} style={{padding: 0}}>
                        <SideNav parentProps={this.props}/>
                    </Col>
                    <Col className={"ll-layout-body-content"} xs={24} sm={24} md={18}>

                        <Navbar className={"body-header-nav"}>
                            <Navbar.Body>
                                <BalanceInfo hasViewPermission={hasBalance} balanceValue={balance} subscriptionUntil={subscriptionUntil} isSubscriptionActive={isSubscriptionActive}/>
                                <BalanceButton hasViewPermission={hasBalance}/>
                                <Nav pullRight className={"ll-body-content-header"}>
                                    <Nav.Item icon={<Icon icon="sign-out" />} onClick={this.handleLogout}>
                                        {language.LOGOUT}
                                    </Nav.Item>
                                </Nav>
                                <Nav pullRight className={"ll-body-content-header"}>
                                    <Nav.Item onClick={() => { /*TODO history.push(pathConstants.FAQ)*/alert("Скоро добавим :)"); }}>
                                        {language.FAQ}
                                    </Nav.Item>
                                </Nav>
                                <Nav pullRight className={"ll-body-content-header"}>
                                    <Nav.Item onClick={() => { history.push(pathConstants.SETTINGS) }}>
                                        {language.SETTINGS}
                                    </Nav.Item>
                                </Nav>
                                <Nav pullRight className={"ll-body-content-header"} style={{display:hasBalance ? 'inherit' : 'none'}}>
                                    <Nav.Item onClick={() => { history.push(pathConstants.BILLS); }}>
                                        {language.BILLS}
                                    </Nav.Item>
                                </Nav>
                            </Navbar.Body>
                        </Navbar>
                        <div className={"ll-body-content-wrapper"}>
                            <Switch>

                                <PrivateRouteView exact path={pathConstants.TESTS_LIST} comp={TestsList}
                                                  viewPermit={hasTestViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.TEST_DETAILS} comp={TestDetails}
                                                  viewPermit={hasTestViewPermit}/>
                                <PrivateRouteView exact path={pathConstants.TEST_CREATE} comp={TestDetails}
                                                  viewPermit={hasTestViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.TOPICS_LIST} comp={TopicsList}
                                                  viewPermit={hasTopicsViewPermit}/>
                                <PrivateRouteView exact path={pathConstants.SECTIONS_LIST} comp={SectionsList}
                                                  viewPermit={hasSectionViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.CANDIDATE_TESTS_LIST} comp={CandidateTestsList}
                                                  viewPermit={hasCandidateTestViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.CANDIDATE_TEST_DETAILS} comp={CandidateTestDetails}
                                                  viewPermit={hasCandidateTestViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.QUESTIONS_LIST} comp={QuestionsList}
                                                  viewPermit={hasQuestionViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.QUESTION_DETAILS} comp={QuestionDetails}
                                                  viewPermit={hasQuestionViewPermit}/>
                                <PrivateRouteView exact path={pathConstants.QUESTION_CREATE} comp={QuestionDetails}
                                                  viewPermit={hasQuestionViewPermit}/>

                                <PrivateRouteView exact path={pathConstants.SETTINGS} comp={UserSettings}
                                                  viewPermit={true}/>

                                <PrivateRouteView exact path={pathConstants.BILLS} comp={OrdersList}
                                                  viewPermit={hasBalance}/>

                                <PrivateRouteView exact path={pathConstants.COMPANY_INFO} comp={CompanyInfo}
                                                  viewPermit={hasBalance}/>

                                <PrivateRouteView exact path={pathConstants.DEPOSIT} comp={DepositScreen}
                                                  viewPermit={hasBalance}/>

                                <PrivateRouteView exact path="/" comp={TestsList}
                                                  viewPermit={hasTestViewPermit}/>

                                <Route component={NotFoundPage} />
                            </Switch>
                        </div>
                    </Col>
                    <Col>
                        <div style={{display: 'block', background: '#F2FAFF',
                            fontSize: 16, fontWeight: 550, textAlign: 'center', paddingBottom: '10px'}}>

                            <a style={{cursor: 'pointer'}} onClick={() => {
                            orderService.downloadOffer().then((blob) => {
                                const objectURL = window.URL.createObjectURL(blob)
                                const link = document.createElement('a');
                                link.href = objectURL;
                                link.download = 'Offer.pdf';
                                document.body.appendChild(link);
                                link.click();
                                document.body.removeChild(link);
                            }); }} >{language.PUBLIC_OFFER}</a>
                        </div>
                    </Col>
                </Row>
            </Grid>
        );
    }
}

const BalanceInfo = (props) => {
    if (props.hasViewPermission === false) {
        return null;
    }
    return (
        <Nav>
            <Nav.Item disabled className={"account-balance"}>
                <div style={{display: 'flex'}}><div>{language.YOUR_BALANCE}</div><div style={{color: '#C17373', marginLeft: 5}}>{props.balanceValue}</div></div>
            </Nav.Item>
            <Nav.Item disabled className={"account-balance"}>
                <Subscription subscriptionUntil={props.subscriptionUntil} isSubscriptionActive={props.isSubscriptionActive}/>
            </Nav.Item>
        </Nav>
    );
};

const Subscription = (props) => {
    if (props.isSubscriptionActive === true)
        return (<div style={{display: 'flex'}}><div>{language.SUBSCRIPTION_UNTIL}</div><div style={{color: '#C17373', marginRight: 50}}>{props.subscriptionUntil}</div></div>);
    else return (<div style={{display: 'flex', color: '#DD0000', marginRight: 50}}><div>{language.SUBSCRIPTION_EXPIRED}</div></div>);
}

const BalanceButton = (props) => {
    if (props.hasViewPermission === false) {
        return null;
    }
    return (
        <Nav className={"ll-body-content-header purchase-account"}>
            <Nav.Item icon={<Icon icon={PurchaseAccount}/>} onClick={() => history.push(pathConstants.DEPOSIT)}/>
        </Nav>
    );
};

function mapStateToProps(state) {
    const { authentication} = state;
    const { user } = authentication;
    return {
        user
    };
}

const connectedSideBarLayout = connect(mapStateToProps)(SideBarLayout);
export { connectedSideBarLayout as SideBarLayout };