import config from 'config';
import { apiConstants } from '../constants/api.constants';
import {authHeader} from "../helpers/auth.header";
import {handle500, handleResponse} from "./response.handler";
import {responseMessages} from "../content/responseMessages";

const base64 = require('base-64');


export const userService = {
    login,
    register,
    registerConfirm,
    logout,
    updateProfile,
    getRelatedUsers,
    createRelatedUser,
    deleteRelatedUser,
    getBalance,
    resetPassword,
    resetPasswordConfirm,
    updateCompanyInfo,
    getCompanyInfo
};

function login(username, password) {
    let auth = 'Basic ' + new Buffer(username + ":" + password).toString('base64');
    const requestOptions = {
        method: 'GET',
        headers: {
            'Content-Type': 'application/json',
            'Authorization': auth
        }
    };
    return fetch(`${config.apiUrl}${apiConstants.USERS_PROFILE}`, requestOptions)
        .then(response => {
            const responseText = response.text();
            if (!response.ok) {
                if (response.status === 401 || response.status === 403) {
                    // auto logout if 401 response returned from api
                    logout();
                    return Promise.reject(responseMessages.INCORRECT_LOGIN);
                }
                const body = responseText;
                const error = (body && body.message) || responseMessages.SERVER_INTERNAL_ERROR;
                return Promise.reject(error);
            }
            let user = JSON.stringify({ username, auth} );
            localStorage.setItem('user', user);
            return responseText.then(response => {
                let jsonResponse = JSON.parse(response);
                if (!response || !jsonResponse || !jsonResponse.data) {
                    logout();
                    return Promise.reject(responseMessages.GET_PROFILE_SERVER_INTERNAL_ERROR);
                }
                if (jsonResponse.data && jsonResponse.data.error){
                    logout();
                    return Promise.reject(jsonResponse.data.error.message);
                }
                localStorage.setItem('profile', JSON.stringify(jsonResponse.data));
                return username;
            });
        });
}

function register(username, password, phone, firstName) {
    const requestOptions = {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({
            "email": username,
            "password": password,
            "phone": phone,
            "firstName": firstName
        })
    };
    return fetch(`${config.apiUrl}${apiConstants.USER_REGISTER}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function resetPassword(username) {
    const requestOptions = {
        method: 'PUT',
        headers: {}
    };
    return fetch(`${config.apiUrl}${apiConstants.USERS_RESET_PASSWORD(username)}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function resetPasswordConfirm(resetCode, password) {
    const requestOptions = {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({
            "password": password
        })
    };
    return fetch(`${config.apiUrl}${apiConstants.USERS_RESET_PASSWORD_CONFIRM(resetCode)}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function registerConfirm(confirm) {
    const requestOptions = {
        method: 'PUT',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.USER_REGISTER_CONFIRM(confirm)}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function logout() {
    // remove user from local storage to log user out
    localStorage.removeItem('user');
    localStorage.removeItem('isSubscriptionActive');
    localStorage.removeItem('profile');
    localStorage.removeItem('companyInfo');
}

function updateProfile(profile) {
    const requestOptions = {
        method: 'PUT',
        headers: authHeader(),
        body: profileToJSONBody(profile)
    };
    return fetch(`${config.apiUrl}${apiConstants.USERS}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function updateCompanyInfo(companyInfo) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: companyInfoToJSONBody(companyInfo)
    };
    return fetch(`${config.apiUrl}${apiConstants.COMPANY_INFO}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function getCompanyInfo() {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.COMPANY_INFO}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function getRelatedUsers() {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.USERS}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function createRelatedUser(user) {
    const requestOptions = {
        method: 'POST',
        headers: authHeader(),
        body: JSON.stringify({
            "email": user.email,
            "password": 'change'
        })
    };
    return fetch(`${config.apiUrl}${apiConstants.USER_REGISTER}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function deleteRelatedUser(userId) {
    const requestOptions = {
        method: 'DELETE',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.USER(userId)}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function getBalance() {
    const requestOptions = {
        method: 'GET',
        headers: authHeader()
    };
    return fetch(`${config.apiUrl}${apiConstants.USER_BALANCE}`, requestOptions)
        .then(response => handleResponse(response,
            null,
            null,
            null,
            null,
            handle500
        ));
}

function profileToJSONBody(profile) {
    let request = {};
    if (!!profile.id) request.id = profile.id;
    if (!!profile.firstName) request.firstName = profile.firstName;
    if (!!profile.lastName) request.lastName = profile.lastName;
    if (!!profile.middleName) request.middleName = profile.middleName;
    if (!!profile.email) request.email = profile.email;
    if (!!profile.phone) request.phone = profile.phone;
    if (!!profile.password) request.password = profile.password;
    return JSON.stringify(request);
}

function companyInfoToJSONBody(companyInfo) {
    let request = {};
    if (!!companyInfo.id) request.id = companyInfo.id;
    if (!!companyInfo.userId) request.userId = companyInfo.userId;
    if (!!companyInfo.typeComp) request.typeComp = companyInfo.typeComp;
    if (!!companyInfo.legalNameComp) request.legalNameComp = companyInfo.legalNameComp;
    if (!!companyInfo.inn) request.inn = companyInfo.inn;
    if (!!companyInfo.kpp) request.kpp = companyInfo.kpp;
    if (!!companyInfo.ogrn) request.ogrn = companyInfo.ogrn;
    if (!!companyInfo.bik) request.bik = companyInfo.bik;
    if (!!companyInfo.paymentAccount) request.paymentAccount = companyInfo.paymentAccount;
    if (!!companyInfo.correspondentAccount) request.correspondentAccount = companyInfo.correspondentAccount;
    if (!!companyInfo.bankName) request.bankName = companyInfo.bankName;
    if (!!companyInfo.legalAddress) request.legalAddress = companyInfo.legalAddress;
    if (!!companyInfo.actualAddress) request.actualAddress = companyInfo.actualAddress;
    if (!!companyInfo.nameCeo) request.nameCeo = companyInfo.nameCeo;
    if (!!companyInfo.namePost) request.namePost = companyInfo.namePost;
    if (!!companyInfo.postAddress) request.postAddress = companyInfo.postAddress;
    return JSON.stringify(request);
}

